// (C) Copyright 2011 Hewlett-Packard Development Company, L.P.
//One Touch Update View
//Initial Version
define(['hp/view/AssociationAddView',
  'hp/view/DialogView',
  'hp/view/FormStateView',
  'hp/core/Router',
  'hpsum/presenter/baseline/BaselinePresenter',
  'hpsum/presenter/node/NodePresenter',
  'hp/services/REST',
  'hp/model/Session',
  'hp/core/Localizer',
  'hp/services/Log',
  'text!hpsumpages/one-touch-update/reboot_confirm.html',
  'jquery',
  'hpsum/view/node/jquery.hpsumInstallables',
  'hpsum/view/node/jquery.hpsumInstallResults',
  'hp/lib/jquery.hpPanels',
  'hp/lib/jquery.hpSelect',
  'lib/wizard',
  'hp/lib/jquery.hpCollapsible',
  'hp/lib/jquery.hpDropMenu',
  'hp/lib/jquery.hpProgressBar',
  'lib/jquery.validate',
  'hp/lib/jquery.hpToggle'],
function(AssociationAddView, DialogView, FormStateView, router, BaselinePresenter, NodePresenter, rest, session, localizer, Log , rebootConfirmHtml) { "use strict";

    var OneTouchUpdateView = (function() {

		var VIEW = '#one-touch-page';      
        var PANELS = '.hp-panels';
        var DELTA_BASE = '#hpsum-home-delta-base';
        var NODES_TABLE = '#nodes';
        var OK = '#hpsum-onetouchupdate-ok';
        var ADD_NODES = '#hpsum-nodes-add';
        var NODES_MESSAGE = '#hpsum-nodes-message';
        var BASELINES_TABLE = '#baselines';
        var ADD_BASELINES = '#hpsum-baselines-add';
        var BASELINES_MESSAGE = '#hpsum-baselines-message';
        var ACTION_DIALOG = '#hpsum-action-dialog';
        var COLLAPSE_FORCE = '#hpsum-otu-install-options-force';
        var COLLAPSE_REBOOT = '#hpsum-otu-install-options-reboot';
        var COLLAPSE_SCHEDULE = '#hpsum-otu-install-options-schedule';
        var COLLAPSE_DEPLOY_INVENOTRY = '#hpsum-otu-deploy-inventory';
        var INTERACTIVE_MODE = '#interactive-mode',
        	NONINTERACTIVE_MODE = '#non-interactive-mode',
        	LOCALHOST_NODE = '#node-localhost',
        	MULTIPLE_NODE = '#node-multiple';
        var ROUTE_PREFIX = '/guided-update/index';
        var INVENTORY_BASELINE_TABLE = '#hpsum-otu-inventory-baseline',
        	INVENTORY_NODE_TABLE = '#hpsum-otu-inventory-node',
        	INSTALLABLES_TABLE = '#hpsum-otu-installables',
        	COMPONENTS_TABLE = '#hpsum-otu-components',
        	INSTALLPROGRESS_TABLE = '#hpsum-otu-install-progress',
        	INSTALLRESULTS_TABLE = '#hpsum-otu-install-results-table',
        	COMPONENT_DIALOG = '#hpsum-component-dialog',
        	SPLASH_DIALOG = '#hpsum-splash-dialog',
        	SPLASH_SPINNER = '#hpsum-splash-spinner',
        	CONFIRM_DIALOG = '#hpsum-confirm-dialog',
        	AUTOMATIC_UPDATE_CONFIRM = '#automatic-confirmation-dialog';
        	
        var STEP_INVENTORY = 0, STEP_REVIEW = 1, STEP_DEPLOY = 2;
		
		
		/** 
		 * Disabling all the features excluding one touch update in offline mode. 
		*/
		
		var guimode = session.getProperty("gui_mode");

        if (guimode === "offline_automatic" || guimode === "offline_interactive") {

				$('#hp-main-menu-control').css('display', 'none');
				$('#hp-search-control').css('display', 'none');
				$('#hp-session-control').css('display', 'none');
				$('#hp-activity-control').css('display', 'none');
        }

        
        /**
         * Constructor
         */
        function OneTouchUpdateView() {
          
          	var associationsNodesTable = null;
          	var associationsBaselinesTable = null;
            var allNodes = [], allBaselines = [];
            var currNodes = [], currBaselines = [];
            var actionDialogContents;
            var bModeInteractive=false, bModeNonInteractive=false, bInProgress=false;
            var baselineIterator, nodeIterator;
            var progressTimer, checkBaselineTimer, bBaselineInventoryDone;
            var componentLogDialogContents, confirmDialogContents;
            var bLoadMulti = false;
            var actiondialog;
            var abortButtonPressed = false;
            var bStartDeploy = false;
            var gnodeNodeSettings = [];
            var modeValue, nodeValue;
            var autoConfDialogContents, autoConfDialog;
            
            function initActionNodeTable() {
				
				//Initialize nodes table in the action dialog
            	associationsNodesTable = $(NODES_TABLE).dataTable({
                    bPaginate : false,
                    bFilter : true,
                    bInfo : false,
                    bSort : false,
                    bRetrieve : false,
			    bAutoWidth : false,
                    bDestroy : true,
                    aaData: currNodes,
                    aoColumns : [
                        {sTitle: localizer.getString("hpsum.menu.node"), mDataProp: 'node'},
                        {sTitle: localizer.getString("hpsum.common.type"),  mDataProp: 'descr'},
                        {mDataProp: 'uri', "bVisible": false},
                        {mDataProp: 'uname', "bVisible": false},
                        {mDataProp: 'pwd', 
                        fnRender: function (oObj) {
                            return '<div class="hp-close"></div>';
                        }, sClass: 'hp-icon', sWidth: 20}]
                });
                
                $(NODES_TABLE + ' .hp-close').click(onDeleteNode);
			$(NODES_MESSAGE).text('');
            }
            
            function initActionBaselineTable() {
            	
            	//Initialize baselines table in the action dialog
            	associationsBaselinesTable = $(BASELINES_TABLE).dataTable({
                    bPaginate : false,
                    bFilter : true,
                    bInfo : false,
                    bSort : false,
                    bRetrieve : false,
			    bAutoWidth : false,
                    bDestroy : true,
                    aaData: currBaselines,
                    aoColumns : [
                        {sTitle: localizer.getString("hpsum.common.baseline"), mDataProp: 'baseline', sDefaultContent:''},
                        {sTitle: localizer.getString("hpsum.common.locationStr"), mDataProp: 'descr', sDefaultContent:''},
                        {mDataProp: 'uri', "bVisible": false},
                        {mDataProp: 'name',
                        fnRender: function (oObj) {
                            return '<div class="hp-close"></div>';
                        }, sClass: 'hp-icon', sWidth: 20}
                    ]
                });
                
                $(BASELINES_TABLE + ' .hp-close').click(onDeleteBaseline);
			$(BASELINES_MESSAGE).text('');
            }
                                    
            function getNodeAssociations(search) {
                var regexp = new RegExp(search, 'i');
                var results =
                    $.grep(allNodes, function(result, index) {
                        // exclude existing
                        if ($.grep(currNodes, function(result2, index2) {
                            return (result2.node === result.node);
                        }).length > 0) {
                            return false;
                        }
                        // check search
                        if (search && search.length > 0) {
                            return (result.node.match(regexp) ||
                                    result.descr.match(regexp));
                        }
                        return true;
                    });
                return ({count: results.length, members: results});
            }
            
            function getBaselineAssociations(search) {
                var regexp = new RegExp(search, 'i');
                var results =
                    $.grep(allBaselines, function(result, index) {
                        // exclude existing
                        if ($.grep(currBaselines, function(result2, index2) {
                            return (result2.uri === result.uri);
                        }).length > 0) {
                            return false;
                        }
                        // check search
                        if (search && search.length > 0) {
                            return (result.baseline.match(regexp) ||
                                    result.descr.match(regexp));
                        }
                        return true;
                    });
                return ({count: results.length, members: results});
            }
            
            function onDeleteNode(event) {
                var row = $(this).parents('tr')[0];
                var association = associationsNodesTable.fnGetData(row);
                
                if(association === null)
                	return;
                	
                currNodes = $.grep(currNodes, function (association2) {
                    return association2.uri !== association.uri;
                });
                associationsNodesTable.fnDeleteRow(row);
                $(NODES_MESSAGE).
                    text('Removed ' + association.node);
                event.stopPropagation();
				
				if (allNodes.length == currNodes.length)
					$(ADD_NODES).attr("disabled","disabled");
				else
					$(ADD_NODES).removeAttr("disabled");
            }
            
            function onDeleteBaseline(event) {
                var row = $(this).parents('tr')[0];
                var association = associationsBaselinesTable.fnGetData(row);
                
                if(association === null)
                	return;
                	
                currBaselines = $.grep(currBaselines, function (association2) {
                    return association2.uri !== association.uri;
                });
                associationsBaselinesTable.fnDeleteRow(row);
                $(BASELINES_MESSAGE).
                    text('Removed ' + association.baseline);
                event.stopPropagation();
				
				if (allBaselines.length == currBaselines.length)
					$(ADD_BASELINES).attr("disabled","disabled");
				else
					$(ADD_BASELINES).removeAttr("disabled");
            }
            
            function onAddNodes() {
                var numAdded = 0;
                new AssociationAddView({
                    title: "Add Nodes",
                    dataTableOptions:  {
                        aoColumns : [
                          {sTitle: localizer.getString("hpsum.menu.node"), mDataProp: 'node', sWidth: 100},
                          {sTitle: localizer.getString("hpsum.common.type"), mDataProp: 'descr', sWidth: 200},
                          {mDataProp: 'uri', bVisible: false},
                          {mDataProp: 'uname', bVisible: false},
                          {mDataProp: 'pwd', bVisible: false}
                        ]
                    },
                    getResults: function (query, handlers) {
                        // simulate error if string is "error"
                        if ('error' === query) {
                            localizer.getString("hpsum.oneTouchUpdate.addBaselinesError");
                        } else {
                            // It is assumed this would be performed on the server side
                            handlers.success(getNodeAssociations(query));
                        }
                    },
                    apply: function (results) {
                        numAdded += results.length;
                        currNodes = currNodes.concat(results);
                        associationsNodesTable.fnAddData(results);
                        associationsNodesTable.fnAdjustColumnSizing();
                        $(NODES_TABLE + ' .hp-close').
                            click(onDeleteNode);
                        $(NODES_MESSAGE).
                            text(numAdded + ' ' + localizer.getString("hpsum.common.added") + ' ');
							
						if (allNodes.length == currNodes.length)
							$(ADD_NODES).attr("disabled","disabled");
						else
							$(ADD_NODES).removeAttr("disabled");
                    }
                });
            }
            
            function onAddBaselines() {
                var numAdded = 0;
                new AssociationAddView({
                    title: localizer.getString("hpsum.oneTouchUpdate.addBaselines"),
                    dataTableOptions:  {
                        aoColumns : [
                          {sTitle: localizer.getString("hpsum.common.baseline"), mDataProp: 'baseline', sWidth: 100, sDefaultContent: ''},
                          {sTitle: localizer.getString("hpsum.common.locationStr"), mDataProp: 'descr', sWidth: 200, sDefaultContent: ''},
                          {mDataProp: 'uri', bVisible: false},
			  {mDataProp: 'name', "bVisible": false}
                        ]
                    },
                    getResults: function (query, handlers) {
                        // simulate error if string is "error"
                        if ('error' === query) {
                            handlers.error(localizer.getString("hpsum.oneTouchUpdate.addBaselinesError"));
                        } else {
                            // It is assumed this would be performed on the server side
                            handlers.success(getBaselineAssociations(query));
                        }
                    },
                    apply: function (results) {
                        numAdded += results.length;
                        currBaselines = currBaselines.concat(results);
                        associationsBaselinesTable.fnAddData(results);
                        associationsBaselinesTable.fnAdjustColumnSizing();
                        $(BASELINES_TABLE + ' .hp-close').
                            click(onDeleteBaseline);
                        $(BASELINES_MESSAGE).
                            text(numAdded + ' ' + localizer.getString("hpsum.common.added") + ' ');
							
						if (allBaselines.length == currBaselines.length)
							$(ADD_BASELINES).attr("disabled","disabled");
						else
							$(ADD_BASELINES).removeAttr("disabled");
                    }
                });
            }
            
            function getAllNodesSuccess(data) {

				allNodes = [];
            	currNodes = [];
            	
            	//add default localhost into the list
            
				
                for (var i=0; i< data.hapi.output_data.nodes.node.length; i+=1) {
                    allNodes.push({
                        node: data.hapi.output_data.nodes.node[i].node_name,
                        descr: data.hapi.output_data.nodes.node[i].type,
                        uri: data.hapi.output_data.nodes.node[i].node_uri,
                        uname: "",
                        pwd:""
                    });
					
		    if(data.hapi.output_data.nodes.node[i].node_name === 'localhost')
		    {
			currNodes.push({
                        node: "localhost",
                        descr: data.hapi.output_data.nodes.node[i].type,
                        uri: "",
                        uname:"",
                        pwd:""
                    });
					}
                }
			
                //Initialize action dialog node table with default value
                initActionNodeTable();
                
                bLoadMulti = true;
				
		if (allNodes.length == currNodes.length)
			$(ADD_NODES).attr("disabled","disabled");
		else
			$(ADD_NODES).removeAttr("disabled");
            }
            
            function getAllNodesError(error) {
                
            }
            
            function getBaselineSuccess(baselines) {
            	
            	allBaselines = [];
            	currBaselines = [];
            	
            	//add default baseline into the list
            	allBaselines.push({
                        baseline: "default",
                        descr: 'default',
                        uri: "default",
			name:"default"
                   	});
                
                //Only add default working directory as default location which is selected for operation
                //if user wants to add more baselines then he can use Add Baselines button to add more   	
                currBaselines.push({
                        baseline: "default",
                        descr: 'default',
                        uri: "default",
                        name:"default"
                   	});
                
                for(var i=0; i < baselines.length; i++)
            	{
            	    allBaselines.push({
                        baseline: baselines[i].baseline_name,
                        descr: baselines[i].baseline_url,
                        uri: baselines[i].baseline_uri,
			name:baselines[i].baseline_name
                    });
                }
                
                //Initialize action dialog baseline table with default value
                initActionBaselineTable();
                
                bLoadMulti = true;
				
		if (allBaselines.length == currBaselines.length)
			$(ADD_BASELINES).attr("disabled","disabled");
		else
			$(ADD_BASELINES).removeAttr("disabled");
            }
            
            function getBaselineError(error) {
                
            }
            
            function loadBaselines() {
            	
            	var handlers =	{
                    			success : function(data) {
                        			getBaselineSuccess(data);
                    		 	},
                    			error : function(errorInfo) {
                        			getBaselineError(errorInfo);
                    			}
                			}
                
                BaselinePresenter.getBaselines(handlers);
            }
            
            function loadNodes() {
            	
            	var handlers =	{
                    			success : function(data) {
                        			getAllNodesSuccess(data);
                    		 	},
                    			error : function(errorInfo) {
                        			getAllNodesError(errorInfo);
                    			}
                			}
            		
        		NodePresenter.getAllNodes(handlers);
            }
            
            function btnNextClicked(i) {                
                switch (i) {
                	case 0:
                        
                        //Retrieve node settings for tpm_state and tpm_bypass
                        initNodeSettings();
                        
                        if(bModeNonInteractive)
                        	return 0; 
                        
                        $("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step-selected");
                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step");
                        
                        return 1;
						break;
					case 1:
                        $("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step-selected");
                        
                        //call analyze if any changes to the selection
                        if($('#step' + i + 'Analyze').hasClass("hp-disabled") == false &&
                        	bModeNonInteractive == false)
                        {
                        	if(hasAnyTpm())
                        	{
                        		
                        		var tpm_bypass = { 
                                                   	hapi: {                                                                                                                                                                   
                                                    	tpm_bypass:'true',                                                                                                                                                                                                                 
                                      				}
                                    		};
                                    		
                                var nodeItem = { node_id: gnodeNodeSettings[0].node_id };
                                    		
                           		NodePresenter.setattribute(nodeItem, tpm_bypass, {
                                               success : function(){
                                               		btnAnalyzeClicked(STEP_REVIEW, true);
                                               },
                                               error : function() {
                                               }
                        		});
							}
                        	else {
                        		//if no changes to component selection, go directly to deploy
                        		btnAnalyzeClicked(STEP_REVIEW, true);
                        	}
							
                        	return -1;
                        }
                        else
                        {
                        	//if TPM is set then use setattributes to set tpm bypass before starting deploy
                        	if(hasAnyTpm())
                        	{
                        		
                        		var tpm_bypass = { 
                                                   	hapi: {                                                                                                                                                                   
                                                    	tpm_bypass:'true',                                                                                                                                                                                                                 
                                      				}
                                    		};
                                    		
                                var nodeItem = { node_id: gnodeNodeSettings[0].node_id };
                                    		
                           		NodePresenter.setattribute(nodeItem, tpm_bypass, {
                                               success : function(){
                                               		initDeploy();
                                               },
                                               error : function() {
                                               }
                        		});
							}
                        	else {
                        		//if no changes to component selection, go directly to deploy
                        		initDeploy();
                        	}
                        }
                        
                        return 2;
						break;
					case 2:
                        $("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step-selected");
                        //handle Reboot functionality here
                        break;
				}
            };
			
			function btnPrevClicked(i) {                
                switch (i) {
					case 1:
                        $("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step-selected");
                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step");
                        return 0;
						break;
					case 2:
						
						$("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step-selected");
                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step");
                        //Handle reset functionality here as the name of the button is changed reset in this step
						initReset();
						return 0;
						
                        $("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step-selected");
                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step");
                        
                        //return 1;
						break;
                    case 3: 
						$("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step-selected");
                        break;
				}
            };
			
			function abortBaselines(step) {
				//not yet implemented in baseline manager.. so not functional
			}
			
			function abortNodes(step) {
				
				var nodes = [];
	        	var uri, index, op_status;
	        	var oNodeTable;
	        		
				if(step == 0){
					//iterate through node inventory table 
					//if node is in inventory state, then only send abort command
					oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
	        		if(oNodeTable)
	        			nodes = oNodeTable.fnGetData();
	        	}
	        	else if(step == 2)
	        	{
	        		oNodeTable = $(INSTALLPROGRESS_TABLE).dataTable();
	            		
	        		if(oNodeTable)
	        			nodes = oNodeTable.fnGetData();
	        	}
	        	
	            //add each node, check the current state and send abort command if require
        		for(var i=0; i < nodes.length; i++)
        		{
        			var list = nodes[i].node_uri.split("/");
					var nodeid = list[4];
					op_status = nodes[i].op_status;
					
					if(op_status === "inprogress")
					{
						NodePresenter.abortNode(nodeid, {
							row: i,
							node_id: nodeid,
							success: function(data){
								if(data.hapi.hcode == 0){
									oNodeTable.fnUpdate('abort_success', this.row, 6);
									oNodeTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.userAborted"), this.row, 2);
									oNodeTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>', this.row, 0);
									
									if(step == STEP_INVENTORY)
									   oNodeTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.nodeInventoryAborted"), this.row, 4);
									else if (step == STEP_DEPLOY)
									   oNodeTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.nodeDeployAborted"), this.row, 4);
								}
								else
								{
									oNodeTable.fnUpdate('abort_error', this.row, 6);
									oNodeTable.fnUpdate(data.hapi.hmessage, this.row, 2);
									oNodeTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">ok</span></div></div>', this.row, 0);
								}
								
								$('#node-progress-' + this.node_id).hpProgressBar(0);
							},
							error: function (errorinfo){
								oNodeTable.fnUpdate('abort_error', this.row, 6);
								oNodeTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.userAbortedError"), this.row, 2);
								oNodeTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', this.row, 0);
							}
						});
					}
				}
			}
			
			//Handle abort functionality here
			function btnAbortClicked(i) {
				
				//Baseline abort is not yet implemented in engine
				abortBaselines(i);
				
                //issue abort command for each node. Based on the result, update the node state column
                abortNodes(i);

                //Disable the next button since we aborted the process.
                disableNextButton(i);
                disableAbortButton(i);
                enableResetButton(i);

                abortButtonPressed = true;
            };
            
            function rebootNodes()
            {
            	//Issue reboot command activate to all the nodes 
                //Later on this needs to be enhanced based on node return code (when it is available in REST API)
				var nodes = [];
	        	var uri, index, op_status;
	        	var oNodeTable;
	        		
				oNodeTable = $(INSTALLPROGRESS_TABLE).dataTable();
            		
        		if(oNodeTable)
        			nodes = oNodeTable.fnGetData();
	        	
	            //for each node, check the current state and send activate command if require
        		for(var i=0; i < nodes.length; i++)
        		{
        			var list = nodes[i].node_uri.split("/");
					var nodeid = list[4];
					op_status = nodes[i].op_status;
					
					if(op_status === "done")
					{
						NodePresenter.rebootNode(nodeid, {
							row: i,
							node_id: nodeid,
							success: function(data){
								if(data.hapi.hcode == 0){
									oNodeTable.fnUpdate(localizer.getString('hpsum.RebootConfirm.node'), this.row, 2);
								}
								else
								{
									oNodeTable.fnUpdate(data.hapi.hmessage, this.row, 2);
								}
							},
							error: function (errorinfo){
								oNodeTable.fnUpdate(localizer.getString('hpsum.RebootConfirm.errorReboot'), this.row, 2);
							}
						});
					}
				}
            }
            
            //Handle reboot functionality here
            function btnFinishClicked(step) {                
                
                if(step != 2)
					return;
				
				var contents = $(rebootConfirmHtml);
                localizer.localizeDom(contents);
                $('#hpsum-otu-reboot-title', contents).text(
                    localizer.getString('hpsum.RebootConfirm.title'));
                
                new DialogView({contents: contents,
                	ok: function () {
                    	rebootNodes();
                    },
                    cancel: function () {
                        return;
                    }
                });
	        };
            
            function btnResetClicked(step) {
                
                if(!$('#step' + step + 'Reset').hasClass("hp-disabled")) {
                    initReset();
                }
			}
			
			function showAnalysisSplash(){
				
				 $(SPLASH_DIALOG).dialog({
			        modal : true,
			        position : "center",
			        draggable : false,
			        resizable : false,
			        show : 'fade',
			        dialogClass : 'hp-splash-dialog',
			        width : 400
			    });
			    
			    $(SPLASH_SPINNER).show();	
			}
			
			function closeAnalysisSplash(){
				$(SPLASH_SPINNER).hide();
				$(SPLASH_DIALOG).dialog('destroy');
			}
			
			function btnAnalyzeClicked(step, bdeploy) {
				
				var nodes = [];
	        	var uri, index, op_status;
	        	var oNodeTable;
	        	
	        	//iterate through node inventory table 
				//if node is in inventory state, then only send abort command
				oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
	        	if(oNodeTable)
	        		nodes = oNodeTable.fnGetData();
	        	
	        	if(nodes.length > 0)
	        	{
	        		var breturn=true;
	        		for(var i=0; i < nodes.length; i++)
        			{
        				var opstatus = nodes[i].op_status;
        				if(opstatus === "done")
        				{
        					breturn = false;
        					break;
        				}
        			}
        			
        			if(breturn)
	        			return;
	        	}
	        	
	        	showAnalysisSplash();
	        	
			var nodesprocessed=0;
	        	//add each node table send analysis command
        		for(var i=0; i < nodes.length; i++)
        		{
        			var list = nodes[i].node_uri.split("/");
					var nodeid = list[4];
					op_status = nodes[i].op_status;
					
					if(op_status === "done")
					{
						var inst = $('#hpsum-otu-node-'+ nodeid +'-installable').data('plugin_hpsumInstallables');
						if(inst == undefined)
						continue;
			 			var componentsList = inst.getComponents();
			 			
			 			var item = {node_id: nodeid};
			 			
			 			oNodeTable.fnUpdate('analyze_inprogress', i, 6);
						nodesprocessed++;
			 
						NodePresenter.analysis(item, componentsList, {
								node_id: nodeid,
                    			success : function(data) {
                        			analyzeSuccess(data, this.node_id);
                        		},
                    			error : function(errorInfo) {
                        			analyzeError(errorInfo);
                    			}
                		});
					}
				}
					if(nodesprocessed == 0) {
						closeAnalysisSplash();
						disableAnalyzeButton(STEP_REVIEW);
						return;
					}
				
				var NodesDoneCount=0;
		        
		        var progressTimer = setInterval(function () {
		        	
		        	oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
	        		if(oNodeTable)
	        			nodes = oNodeTable.fnGetData();
	        		
		        	//start inventory for each baseline
            		for(var i=0; i < nodes.length; i++)
            		{
            			var uri = nodes[i].node_uri;
            			var vals = uri.split("/");
            			var nodeid = vals[4];
            			var index = i;
            			var op_status = nodes[i].op_status;
            			
            			//if current operation status is started, then only start the inventory, otherwise no need to start inventory
            			if(op_status === "analyze_inprogress") {
	            				
				        	rest.getURI(uri + "/getstatus", {
		                			row: index,
		                			node_id: nodeid,
		                			success : function(data) {
		                				
		                				if(data.hapi.node_status.node_state === "ANALYSIS")
		                				{
		                					
		                				}
		                				else if(data.hapi.node_status.node_state === "UAN")
		                				{
		                					
		                					oNodeTable.fnUpdate('done', this.row, 6);
		                					
		                					NodesDoneCount++;
		                				}
		                				else if(data.hapi.node_status.node_state === "ERROR")
		                				{
		                					
		                					oNodeTable.fnUpdate('done', this.row, 6);
		                					
		                					NodesDoneCount++;
		            					}
		            					else if(data.hapi.node_status.node_state === "PENDING_FW_UPDATE")
		                				{
		                					
		                					oNodeTable.fnUpdate('done', this.row, 6);
		                					
		                					NodesDoneCount++;
		            					}
		            				},
		                			error : function(errorInfo) {
		                    			
		                			}
		            			});
		            		}
		            	}
		            	
		            	if(NodesDoneCount >= nodes.length) {
		            		
		            		//stop the timer
	            			clearInterval(progressTimer);
	            			
	            			//show install review for all the nodes
					        initInstallReview(bdeploy);
						    
					        closeAnalysisSplash();
					        
					        //disable analyze button
					        disableAnalyzeButton(STEP_REVIEW);
			        	}
            			
		        }, 3000);
			}
			
			function analyzeSuccess(data, nodeid) {
				
			}
			
			function analyzeError(error) {
				$(SPLASH_SPINNER).hide();
				$(SPLASH_DIALOG).dialog('destroy');
			}
			
			function hideAbortButton(step) {
				$('#step' + step + 'Abort').hide();
			}
			
			function enableAbortButton(step) {
				$('#step' + step + 'Abort').removeClass("hp-button hp-secondary hp-disabled").addClass("hp-button hp-secondary");
			}
			
			function disableAbortButton(step) {
				$('#step' + step + 'Abort').removeClass("hp-button hp-secondary").addClass("hp-button hp-secondary hp-disabled");
			}
			
			function enableNextButton(step) {
				$('#step' + step + 'Next').removeClass("hp-button hp-secondary hp-disabled").addClass("hp-button hp-primary");
			}
			
			function disableNextButton(step) {
				$('#step' + step + 'Next').addClass("hp-button hp-secondary hp-disabled");
			}
			
			function enableResetButton(step) {
				$('#step' + step + 'Reset').removeClass("hp-button hp-secondary hp-disabled").addClass("hp-button hp-secondary");
			}
			
			function disableResetButton(step) {
				$('#step' + step + 'Reset').addClass("hp-button hp-secondary hp-disabled");
			}
			
			function enableBackButton(step) {
				$('#step' + step + 'Prev').removeClass("hp-button hp-secondary hp-disabled").addClass("hp-button hp-secondary");
			}
			
			function disableBackButton(step) {
				$('#step' + step + 'Prev').removeClass("hp-button hp-secondary").addClass("hp-button hp-secondary hp-disabled");
			}
			
			function enableAnalyzeButton(step) {
				$('#step' + step + 'Analyze').removeClass("hp-button hp-secondary hp-disabled").addClass("hp-button hp-secondary");
				
				//Disable Deploy button if analyze button is enabled and vice versa
				//disableNextButton(step);
				enableNextButton(step);
			}
			
			function disableAnalyzeButton(step) {
				$('#step' + step + 'Analyze').addClass("hp-button hp-secondary hp-disabled");
				
				//enable Deploy button if analyze button is disabled and vice versa
				//before enabling Deploy button, make sure no failed dependency components are selected.. TODO
				//enableNextButton(step);
			}
						
			function enableRebootButton(step) {
				$('#step' + step + 'Reboot').removeClass("hp-button hp-secondary hp-disabled").addClass("hp-button hp-secondary");
			}
			
			function disableRebootButton(step) {
				$('#step' + step + 'Reboot').removeClass("hp-button hp-secondary").addClass("hp-button hp-secondary hp-disabled");
			}
			
			function hideAllButtons(step) {
				var parent = $('#step' + step + 'commands');
				
				//Only show the Abort and Reset buttons and hide all other buttons
				parent.find('.hp-button').each(function() {				    
				    if(!($(this).attr('id').match(/abort/i) || $(this).attr('id').match(/reset/i)))
				        $(this).hide();				    
				});
			}
			
			function showAllButtons(step) {
				var parent = $('#step' + step + 'commands');
				
                parent.find('.hp-button').each(function() {
                    $(this).show();
                });
			}
			
			function showDeployButton(step) {
				$('#step' + step + 'commands').show();
				enableNextButton(step);
				enableResetButton(step);
				$('#step' + step + 'Prev').hide();
				$('#step' + step + 'Analyze').hide();
			}
			
			function showWizardStep(step) {
				$('#step' + step).show();
			}
			
			function hideWizardStep(step) {
				$('#step' + step).hide();
			}
            
			function validateStep(i) {
                switch (i) {
                    case 1:  /*alert('This is validated by index'); */return true; break;
                    default:
                        return true;
                        break;
 
                }
            };
	
			function validateByTitle(title) {
				switch (title) {
	                case 'test':  /*alert('This is validated by title');*/ return true; break;
					default:
	                       return true;
	                       break;
                }
            };
            
            function showActionDialog() {
                
            	//show action dialog
            	actiondialog = new DialogView({contents: actionDialogContents,
                    ok: function (elem) {
                            modeValue = $('input[name="mode-option"]:checked').val();
                            nodeValue = $('input[name="node-option"]').val();
                            
                            if(modeValue === "non-interactive-mode") {
                                autoConfDialog = new DialogView({contents: autoConfDialogContents,
                                    ok: function(elem){
                                        startOneTouchUpdate();
                                    },
                                    cancel: function () {
                                        initReset();
                                    }
                                });
                            } else {
                                startOneTouchUpdate();
                            }
                    },
                    cancel: function () {
                        router.go('/welcome', 'canceled guided update');
                    }
                });



                actiondialog.setMessage({summary: localizer.getString("hpsum.oneTouchUpdate.selectModeTypeDeployment"),
                    status: "ok",
                    details: localizer.getString("hpsum.oneTouchUpdate.moreDetails"),
                    actions: ""/*[linkTargetBuilder.makeLink('Edit device',
                      '/device/item1', 'edit', 'device')]*/
                }, true);
            }
            
            function indexLocation(data) {
            	
            	var i, uri;
            	
            	var oTable = $(INVENTORY_BASELINE_TABLE).dataTable();
            	
            	//iterate through baselines and add them into baselines wizard table
            	for(i = 0; i < data.hapi.output_data.baselines.baseline.length; i += 1)
	        	{
	        		var item = [{status: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>', 
        			baseline_name: data.hapi.output_data.baselines.baseline[i].baseline_name,
        			baseline_status: localizer.getString("hpsum.oneTouchUpdate.baselineAdded"),
        			baseline_progress: '<div id="baseline-progress' + i + '" style="height:15px"></div>',
        			baseline_details: '',
        			baseline_uri: data.hapi.output_data.baselines.baseline[i].baseline_uri,
        			op_status: "started"}];
        			
        			//add baseline into baseline inventory table of step1 of wizard	    
					oTable.fnAddData(item);
					
					$('#baseline-progress' + i).hpProgressBar(0);
				}
				
				baselineIterator -= baselineIterator;
			}
            
            function onaddLocationSuccess(location, data) {
            	
            	if(data.hapi.hcode != 0) {
            		onaddLocationError(location, data.hapi.hmessage);
            		return;
            	}
            	
            	//process the output and request for inventory of location
            	var location_id = data.hapi.command.output_data.location_id;
            	var uri = "/Session/" + session.getToken() + "/Baseline/location/" + location_id + "/inventory";
				
				rest.getURI(uri, {
                			success : function(dataLocSuccess) {
                				//using timer here as sometimes index command will not provide any output if inventory is issued before
                				//inventory command take some time to get the result
                				//so checking until we get something out of index command  			
                				var timer = setInterval( function () {
                					uri = "/Session/" + session.getToken() + "/Baseline/location/" + location_id + "/index";
                    				rest.getURI(uri, 
                    							{success: function(dataIndex) {
                    								//if some baselines exists
                    								if(dataIndex.hapi.output_data.baselines.baseline.length > 0) {
                    									indexLocation(dataIndex);
                    									clearInterval(timer);
                    								}
													else
													{
														clearInterval(timer);
														var oProgressTable = $(INVENTORY_NODE_TABLE).dataTable();
	            										oProgressTable.fnUpdate('', 0, 7);
					
														oProgressTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', 0, 0);
														oProgressTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.failedToAddBaseline"), 0, 2);
														oProgressTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.noBaselineDefault"), 0, 4)
														oProgressTable.fnUpdate('done', 0, 6);
														handleExpressExit();
													}
                    								//otherwise also check whether inventory is completed or still in progress
                    							},
                    							error: function(error) {
                    								clearInterval(timer);
                    								handleExpressExit();
                    							}
                    						}
                    				);
                    				
                    			}, 3000);
                		 	},
                			error : function(errorInfo) {
                    			onaddBaselineError(location_id, errorInfo);
                    			handleExpressExit();
                			}
            	});
			}
            
            function onaddLocationError(location_id, error) {
            	//handle error condition
            	var oTable = $(INVENTORY_BASELINE_TABLE).dataTable();
								
			    oTable.fnClearTable();
			    
			    var item = [{status: '<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', 
    			baseline_name: location_id,
    			baseline_status: localizer.getString("hpsum.oneTouchUpdate.failedToAddBaseline"),
    			baseline_progress: '',
    			baseline_details: error,
    			baseline_uri: location_id,
    			op_status: 'error'}];

				//add baseline into baseline inventory table of step1 of wizard	    
				oTable.fnAddData(item);
				handleExpressExit();
            }
            
            function onaddNodeSuccess(node_name, node_type, data) {
            	
            	//process the output and request for inventory of location
            	var oTable = $(INVENTORY_NODE_TABLE).dataTable();
								
			    var item = [{status: '<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>', 
                			node_name: node_name,
                			node_status: localizer.getString("hpsum.oneTouchUpdate.addedNode"),
                			node_progress: '<div id="node-progress-' + data.hapi.node_id + '" style="height:15px"></div>',
                			node_details: '',
                			node_uri: "/Session/" + session.getToken() + "/node/" + data.hapi.node_id,
                			op_status: "started",
                			progress: 0,
                			node_type: node_type}];
				
				//add node into node inventory table of step1 of wizard	    
				oTable.fnAddData(item);
				
				$('#node-progress-' + data.hapi.node_id).hpProgressBar(0);
					NodePresenter.setNodeForAdvOptions({node_name: node_name, node_id: data.hapi.node_id});
				
				nodeIterator -= nodeIterator;
			}
            
            function onaddNodeError(node_name, error) {
            	//handle error condition
            	var oTable = $(INVENTORY_NODE_TABLE).dataTable();
								
			    var item = [{status: '<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', 
                			node_name: node_name,
                			node_status: localizer.getString("hpsum.oneTouchUpdate.failedToAddNode"),
                			node_progress: '',
                			node_details: localizer.getString("hpsum.common.error"),
                			node_uri: node_name,
                			op_status: 'error',
                			progress: 0}];

				//add node into node inventory table of step1 of wizard	    
				oTable.fnAddData(item);
				handleExpressExit();
            }
            
            function startOneTouchUpdate() {
            	
		var error_condition = false; 
            	if(bInProgress == true)
            		return;
            	
            	bStartDeploy = false;
            		
            	//get the selected mode, node option, baselines, nodes
            	var nodes = [], baselines = [];
            	            	
            	baselineIterator=0, nodeIterator=0;
            	
            	//Flag is set to indicate one touch update is in progress
            	bInProgress = true;
            	//Initialize the wizard steps
            	showWizardStep(STEP_INVENTORY);
            	hideWizardStep(STEP_REVIEW);
            	hideWizardStep(STEP_DEPLOY);
            	
            	//Disable next button by default till inventory is done
            	disableNextButton(STEP_INVENTORY);
            	//disable abort button, enable when inventory is in progress
            	disableAbortButton(STEP_INVENTORY);
            	//disable reset button, enable after inventory is completed
            	disableResetButton(STEP_INVENTORY);
            	
            	$('#hpsum-otu-advance-action').removeClass('hp-disabled');
            	
            	bBaselineInventoryDone = false; //update this to true after all baselines inventory is done
            	
            	var guimode = session.getProperty("gui_mode");
            	//guimode is not defined
				
				if(modeValue != undefined)
				{
					if(modeValue === "interactive-mode")
	            	{
	            		bModeInteractive = true;
	            		bModeNonInteractive = false;
	            		
	            		showAllButtons(STEP_INVENTORY);
	            	}
	            	else if(modeValue === "non-interactive-mode")
	            	{
	            		bModeInteractive = false;
	            		bModeNonInteractive = true;
	            		
	            		hideAllButtons(STEP_INVENTORY);
	            	}
				}
				else if(guimode && modeValue === undefined) 
            	{ 
            		//add default localhost into the list
            		nodes.push({
                        node: "localhost",
                        descr: '',
                        uri: "",
                        uname:"",
                        pwd:""
                    });
                    
                    //add default baseline into the list
            		baselines.push({
                        baseline: "default",
                        descr: 'default',
                        uri: "default",
			name:"default"
                   	});
                    
					if(guimode === "offline_automatic" || guimode === "express")
					{
						bModeNonInteractive = true;
						bModeInteractive = false;
						
						hideAllButtons(STEP_INVENTORY);
					}
					else if (guimode === "offline_interactive")
					{
						bModeNonInteractive = false;
						bModeInteractive = true;
						showAllButtons(STEP_INVENTORY);
					}
				}
	            
	            if(bLoadMulti)
	            {
	            	if(nodeValue === "node-localhost") {
	            		//add localhost into the nodes list
	            		nodes = [];
	            		nodes.push({
	                        node: "localhost",
	                        descr: '',
	                        uri: "",
	                        uname:"",
	                        pwd:""
	                    });
	                    
	                    //add default baseline
	                    baselines = [];
	                    baselines.push({
	                        baseline: "default",
	                        descr: 'default',
	                        uri: "default",
				name:"default"
	                   	});
	            	}
		            else 
		            {
		        		var oBaselineTable = $(BASELINES_TABLE).dataTable();
		        	 	var oNodeTable = $(NODES_TABLE).dataTable();
		        	 	
		        	 	if(oNodeTable)
		        	 		nodes = oNodeTable.fnGetData();
		        	 	if(oBaselineTable)
		        	 		baselines = oBaselineTable.fnGetData();
		        	 }
	        	}
	        	
	        	var oTable = $(INVENTORY_BASELINE_TABLE).dataTable();
            	oTable.fnClearTable();
            	
            	oTable = $(INVENTORY_NODE_TABLE).dataTable();
            	oTable.fnClearTable();
            	
            	//add all the baselines and node
                var ind;
                for(ind=0; ind < baselines.length; ind++) 
                {
                	var uri = baselines[ind].uri;
                	//if the baseline is not already added, then only add it, otherwise just add it to the table and check the status and issue inventory
                	if(uri === "default")
                	{
	                	baselineIterator++;
	                	rest.postObject("Session/" + session.getToken() + "/Baseline/add", 
									{
									 hapi: { uri: baselines[ind].uri, locationtype: 'local' }
									},  "", 
	            		         {
	            		         	location: uri,
	                    			success : function(data) {
	                        			onaddLocationSuccess(this.location, data);
	                    		 	},
	                    			error : function(errorInfo) {
	                        			onaddLocationError(baselines[ind].baseline, errorInfo);
	                    			}
	                			}, {});
	                }
	                else 
	                {
	                	var item = [{status: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>', 
	        			baseline_name: baselines[ind].baseline,
	        			baseline_status: localizer.getString("hpsum.oneTouchUpdate.baselineAdded"),
	        			baseline_progress: '<div id="baseline-progress' + ind + '" style="height:15px"></div>',
	        			baseline_details: '',
	        			baseline_uri: baselines[ind].uri,
	        			op_status: "started"}];
	        			
	        			var oTmpTable = $(INVENTORY_BASELINE_TABLE).dataTable();
	        			
	        			//add baseline into baseline inventory table of step1 of wizard	    
						oTmpTable.fnAddData(item);
						
						$('#baseline-progress' + ind).hpProgressBar(0);
	                }
                }
                
                //add nodes
                for(ind=0; ind < nodes.length; ind++) 
                {
                	nodeIterator++;
                	var nodename = nodes[ind].node;
                	var nodetype = nodes[ind].descr;
                	var uri = nodes[ind].uri;
                	var item;
                	
                	//for localhost or default node
                	if(uri === "")
                	{
                		item = {ip:nodes[ind].node, name: nodes[ind].node, type:'unknown', use_current_credentials: true};
                	}
                	else
                	{
                		//node already added in the node screen
                		item = {ip:nodes[ind].node, name: nodes[ind].node};
                	}
                	setTimeout(function(){			
		                	rest.postObject("Session/" + session.getToken() + "/node/add", 
									{
									 hapi:item
									},  "", 
			        		        {
			        		        	node_name: nodename,
			        		        	node_type: nodetype,
			                			success : function(data) {
			                    			onaddNodeSuccess(this.node_name, this.node_type, data);
			                		 	},
			                			error : function(errorInfo) {
			                    			onaddNodeError(this.node_name, errorInfo);
			                			}
			            			}, {});
					},250);
               	} 
                var baselinesAdded = [], nodesAdded = [] , baselinesList = [];
                var remaining = 100;
                var bBaselinesDoneCount=0;
                var bNodesDoneCount=0,  baselineCount = 0;
                //check the progress of inventory operation every 2 secs
                //progressTimer = setInterval(checkProgress(), 2000);
				var oBaselineTable = $(INVENTORY_BASELINE_TABLE).dataTable();
				if(oBaselineTable)
	            			baselines = oBaselineTable.fnGetData();
					for(var i=0; i < baselines.length; i++)
					{
						var uri = baselines[i].baseline_uri;
						var	sub = uri.search("/location/");
						baselinesList[baselineCount++] = uri.substr(sub);
					}
                progressTimer = setInterval(function () {
                	
                	if(bModeNonInteractive)
                		hideAllButtons(STEP_INVENTORY);
                		
                	if(baselineIterator === 0 && bBaselineInventoryDone === false)
	            	{
	            		var oBaselineTable = $(INVENTORY_BASELINE_TABLE).dataTable();
	            		var uri, index, sub, op_status;
	            		
	            		if(remaining > 10)
		                	remaining -= 10;
	            		
	            		if(oBaselineTable)
	            			baselinesAdded = oBaselineTable.fnGetData();
	            		//start inventory for each baseline
	            		for(var i=0; i < baselinesAdded.length; i++)
	            		{
	            			uri = baselinesAdded[i].baseline_uri;
	            			index = i;
	            			op_status = baselinesAdded[i].op_status;
							
	            			//if currest operation status is started, then only start the inventory, otherwise no need to start inventory
	            			if(op_status === "started") {
	            				
	            				if(oBaselineTable){
				                	oBaselineTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', index, 0);
				                	oBaselineTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryInProgress"), index, 2);
				                	oBaselineTable.fnUpdate('inprogress', index, 6);
				                	$('#baseline-progress' + index).hpProgressBar(1);
				                }
				                
		            			rest.getURI(baselinesAdded[i].baseline_uri + "/inventory", {
		            				row: index,
	                    			success : function(data) {
	                    				
	                    				var oTmpTable = $(INVENTORY_BASELINE_TABLE).dataTable();
	                    				//update status
	                    				if(oTmpTable){
						                	oTmpTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', this.row, 0);
						                	oTmpTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryInProgress"), this.row, 2);
						                	oTmpTable.fnUpdate('inprogress', this.row, 6)
						                }
						                
						                //update progress
						                $('#baseline-progress' + this.row).hpProgressBar(1);
						            },
	                    			error : function(errorInfo) {
	                        			//checkBaselineInventoryError(uri, index, errorInfo);
	                        			handleExpressExit();
	                    			}
		                		});
		                	}
		                	else if(op_status === "inprogress") {
		                		
		                		rest.getURI(uri + "/getstatus", {
		                			row: index,
                    				success : function(data) {
                    					//get the row index for the applicable baseline
	                    				if(data.hapi.baseline_state === "inventory_in_progress" || data.hapi.baseline_state === "manifest_component_processing")
	                    				{
	                    					var currprogress = (data.hapi.baseline_state_detail.current_progress/data.hapi.baseline_state_detail.total_no_component) * 100;
	                    					$('#baseline-progress' + this.row).hpProgressBar(currprogress);
						                	
	                    					oBaselineTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', this.row, 0);
	                    					oBaselineTable.fnUpdate( localizer.getString("hpsum.baseline.show.inventoryInProgress"), this.row, 2);
	                    					var detail = data.hapi.baseline_state_detail.detail + " (" + data.hapi.baseline_state_detail.current_progress + "/" + data.hapi.baseline_state_detail.total_no_component + ")";
	                    					oBaselineTable.fnUpdate(detail, this.row, 4);
	                    					oBaselineTable.fnUpdate('inprogress', this.row, 6)
	                    				}
   	                 					else if(data.hapi.baseline_state === "inventory_completed")
	                    				{	
											var i = 0,j = 0,k = 0;
	                    					$('#baseline-progress' + this.row).hpProgressBar(100);
											var uri_new = '/location/' + data.hapi.location_id + '/bundle/' + data.hapi.baseline_id;
											for(i = 0; i < baselinesList.length ; i++)
											{
												if( baselinesList[i] == uri_new )
												{
													for(j = i, k = i+1; k <= baselineCount ; j++,k++)
													{
														baselinesList[j] = baselinesList[k];
													}
													baselineCount--;
												}
											}
											
	                    					oBaselineTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>', this.row, 0);
	                    					oBaselineTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryCompleted"), this.row, 2);
	                    					oBaselineTable.fnUpdate(localizer.getString("hpsum.baseline.show.totalComponents") + ' ' + data.hapi.baseline_state_detail.total_no_component, this.row, 4);
	                    					oBaselineTable.fnUpdate('done', this.row, 6)
	                    					
	                    					bBaselinesDoneCount++;
	                    				}
	                    				else if(data.hapi.baseline_state === "inventory_needed")
                    					{
                    						oBaselineTable.fnUpdate('started', this.row, 6)
                    					}
                    				},
                    				error : function(errorInfo) {
                    					handleExpressExit();
                        			}
                				});
		                		
		                	}
		                }
	            		
	            		if((bBaselinesDoneCount >= baselinesAdded.length) && (!baselineCount) )
	            			bBaselineInventoryDone = true;
	            	}
	            	
	            	//check if all the nodes are added successfully, we can carry on with the inventory operation
	            	//This can be started after completing the baseline inventory
	            	if(nodeIterator === 0 && bBaselineInventoryDone)
	            	{
	            		var oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
	            		var uri, index, op_status, node_id;
	            		
	            		if(oNodeTable)
	            			nodesAdded = oNodeTable.fnGetData();
	            		//start inventory for each baseline
	            		for(var i=0; i < nodesAdded.length; i++)
	            		{
	            			uri = nodesAdded[i].node_uri;
	            			var vals = uri.split("/");
	            			node_id = vals[4];
	            			index = i;
	            			op_status = nodesAdded[i].op_status;
	            			var currprogress = nodesAdded[i].progress;
	            			//if current operation status is started, then only start the inventory, otherwise no need to start inventory
	            			if(op_status === "started") {
	            				//get the available baselines and pass it to node inventory
	            				var itemBaselines = {baselines: {baseline:[]}};
	            				for(var j=0; j< baselinesAdded.length; j++)
	            				{
	            					var baselineid, paths;
	            					paths = baselinesAdded[j].baseline_uri.split("/");
	            					baselineid = paths[5] + "/" + paths[7];
	            					itemBaselines.baselines.baseline.push(baselineid);
	            				}
	            				
	            				oNodeTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', index, 0);
	            				oNodeTable.fnUpdate(localizer.getString("hpsum.baseline.nodeInventoryInProgress"), index, 2);
			                	$('#node-progress-'+ node_id).hpProgressBar(1);
			                	oNodeTable.fnUpdate('inprogress', index, 6);
			                	
			                	rest.postObject(uri + "/inventory", 
												{ hapi:itemBaselines },
												"", 
					            		         { uri: nodesAdded[i].node_uri,
					            		           nodeid: node_id,
					            		           row: index,
		                    			success : function(data) {
		                    	
		                    				enableAbortButton(STEP_INVENTORY);
		                    							
		                    				var oTable = $(INVENTORY_NODE_TABLE).dataTable();
		                    				
		                    				//update status
		                    				if(oTable){
							                	oTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', this.row, 0);
							                	oTable.fnUpdate(localizer.getString("hpsum.baseline.nodeInventoryInProgress"), this.row, 2);
							                	oTable.fnUpdate('inprogress', this.row, 6);
							                }
							                
							                //move progress bar
							                $('#node-progress-'+ this.nodeid).hpProgressBar(1);
							                oTable.fnUpdate(1, this.row, 7);
							                
							                if(data.hapi.hcode != 0) {
							                	oTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', this.row, 0);
							                	oTable.fnUpdate(data.hapi.hmessage, this.row, 4);
							                	oTable.fnUpdate(localizer.getString("hpsum.baseline.nodeInventoryFailed"), this.row, 2);
							                	
							                	oTable.fnUpdate('done_error', this.row, 6);
							                	$('#node-progress-' + this.nodeid).hpProgressBar(100);
							                	oTable.fnUpdate(100, this.row, 7);
							                	
							                	bNodesDoneCount++;
							                }
								             
							            	//Not required. Will remove later after verifying    
		                        			//checkNodeInventorySuccess(uri, index, data);
		                    		 	},
		                    			error : function(errorInfo) {
		                        			checkNodeInventoryError(this.uri, this.row, errorInfo);
								handleExpressExit();
		                    			}
		                			}, {});
                			}
                			else if (op_status === "inprogress") {
                				
                				rest.getURI(uri + "/getstatus", {
                					uri: nodesAdded[i].node_uri,
					            	nodeid: node_id,
					            	row: index,
					            	progress: currprogress,
	                    			success : function(data) {
	                    				
	                    				//get the row index for the applicable node
	                    				var oTable = $(INVENTORY_NODE_TABLE).dataTable();
	                    				
	                    				oTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', this.row, 0);
	                    				
	                    				if(data.hapi.node_status.node_state === "INVENTORY")
	                    				{
	                    					if(this.progress < 50) 
	                    					{
		                    					$('#node-progress-' + data.hapi.node_status.node_id).hpProgressBar(this.progress+2);
		                    					oTable.fnUpdate(this.progress+2, this.row, 7);
		                    				}
		                    				oTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', this.row, 0);
						                	oTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryInProgress"), this.row, 2)
						                	oTable.fnUpdate(data.hapi.node_status.node_status, this.row, 4);
						                	oTable.fnUpdate('inprogress', this.row, 6);
	                    				}
	                    				else if(data.hapi.node_status.node_state === "SELFINVENTORY")
	                    				{
	                    					if(this.progress < 95) 
	                    					{
		                    					$('#node-progress-' + data.hapi.node_status.node_id).hpProgressBar(this.progress+2);
		                    					oTable.fnUpdate(this.progress+2, this.row, 7);
		                    				}
		                    				oTable.fnUpdate('<div class="hp-status hp-changing"><div class="hp-status-changing"></div></div>', this.row, 0);
		                    				oTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryInProgress"), this.row, 2)
						                	oTable.fnUpdate(data.hapi.node_status.node_status, this.row, 4);
						                	oTable.fnUpdate('inprogress', this.row, 6);
	                    				}
	                    				else if(data.hapi.node_status.node_state === "UAN")
	                    				{
	                    					$('#node-progress-'+ data.hapi.node_status.node_id).hpProgressBar(100);
						                	oTable.fnUpdate(100, this.row, 7);
						                	
	                    					oTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>', this.row, 0);
	                    					oTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryCompleted"), this.row, 2);
											if(bModeInteractive)
												oTable.fnUpdate(data.hapi.node_status.node_status, this.row, 4);
											else
												oTable.fnUpdate('', this.row, 4);
	                    					oTable.fnUpdate('done', this.row, 6);
	                    					
	                    					bNodesDoneCount++;
											//getupdatelist to check the number of components											
											NodePresenter.getUpdateList(data.hapi.node_status.node_id, {                        
												success: function(data) {
													var compcount=0;
													for(var i=0; i< data.hapi.baselines.baseline.length; i++)
													{
														if (data.hapi.baselines.baseline[i].hasOwnProperty("components"))
														{
															compcount += data.hapi.baselines.baseline[i].components.component.length;														
													    }
													}														

													if(compcount == 0)
														error_condition = true;
												},
												error: function(data) {
													handleExpressExit();
												}
											});
		            					}
	                    				else if(data.hapi.node_status.node_state === "ERROR")
	                    				{
	                    					$('#node-progress-'+ data.hapi.node_status.node_id).hpProgressBar(100);
						                	oTable.fnUpdate(100, this.row, 7);
						                	
	                    					oTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', this.row, 0);
	                    					oTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryError"), this.row, 2);
	                    					oTable.fnUpdate(data.hapi.hmessage, this.row, 4);
	                    					oTable.fnUpdate('done_error', this.row, 6);
	                    					error_condition = true;
	                    					bNodesDoneCount++;
	                    				}
	                    				else if(data.hapi.node_status.node_state === "PENDING_FW_UPDATE")
	                    				{
	                    					$('#node-progress-'+ data.hapi.node_status.node_id).hpProgressBar(100);
						                	oTable.fnUpdate(100, this.row, 7);
						                	
	                    					oTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', this.row, 0);
	                    					oTable.fnUpdate(localizer.getString("hpsum.baseline.show.inventoryError"), this.row, 2);
	                    					oTable.fnUpdate(localizer.getString("hpsum.integrityilo.activateOnNodeScreen"), this.row, 4);
	                    					oTable.fnUpdate('done_error', this.row, 6);
	                    					
	                    					bNodesDoneCount++;
	                    				}
		                    		},
	                    			error : function(errorInfo) {
	                    					handleExpressExit();
	                        		}
	                			});
                			}
                			else if (op_status === "abort_error" || op_status === "abort_success") {
                				$('#node-progress-'+ node_id).hpProgressBar(100);
                				oNodeTable.fnUpdate(100, index, 7);
                				bNodesDoneCount++;
                			}
		                }
	            		
	            		if(bNodesDoneCount >= nodesAdded.length) {
	            			//stop the timer once everything is done
	            			clearInterval(progressTimer);
	            			
	            			disableAbortButton(0);
	            			
                            
                            if (abortButtonPressed) {
                                // Clear this indicator so that if startOneTouchUpdate gets called again,
                                // it won't think the abort button was pressed for the current iteration.
                                abortButtonPressed = false;
                            }
                            // The process was not aborted so get ready for the next step.
                            else  {
	            			    enableNextButton(STEP_INVENTORY);
	            			    enableResetButton(STEP_INVENTORY);
								//Enable Actions
								$('#hpsum-otu-actions').show();
								if(error_condition == true)
								{
									disableRebootButton(STEP_DEPLOY);
									disableNextButton(STEP_INVENTORY);
									handleExpressExit();
								}
                                //go to deploy step
                                if(bModeNonInteractive && error_condition == false) {					
								
									$("#step0Next").click();					
                                }

                            }
                        }
	            	}
		            	
	            	}, 3000);
                  
            }
            
            function onSelectionChanged(id, value, componentList){
				if(value == true){
					enableAnalyzeButton(STEP_REVIEW);
					enableNextButton(STEP_REVIEW);
				}

                if (componentList.failed_dependency > 0)
                    bStartDeploy = false;
				
				if(componentList.component.length > 0 && componentList.failed_dependency == 0) {
					enableNextButton(STEP_REVIEW);
					if(bStartDeploy) {
						showWizardStep(STEP_DEPLOY);
						hideWizardStep(STEP_REVIEW);
						$("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
						$("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
						$("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step-selected");
						initDeploy();
					}
				} else {
					disableNextButton(STEP_REVIEW);  
				}
				
				toggleButtons(id, STEP_REVIEW);
			}
            
            function initInstallReview(bdeploy) {
            	var oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
        		var nodes = [];
        		var uri, index, op_status;
        		var bNodesDoneCount=0;
        		
        		bStartDeploy = bdeploy;
        		
        		//disable deploy button by default. Enable only if some components are ready for installation
        		disableNextButton(STEP_REVIEW);
        		//disable analyze button by default. Enable only if component selection or force option changed
        		disableAnalyzeButton(STEP_REVIEW);
			showWizardStep(STEP_REVIEW);
			hideWizardStep(STEP_INVENTORY);
			hideWizardStep(STEP_DEPLOY);
				
				
			//hide the tpm status on startup
			$('#tpm-status-update').hide();
        		
        		$('#all-installables').empty();
        		
        		if(oNodeTable)
        			nodes = oNodeTable.fnGetData();
	            //add each node and its installable components
        		for(var i=0; i < nodes.length; i++)
        		{
        			var list = nodes[i].node_uri.split("/");
					var nodeid = list[4];
					var op_status = nodes[i].op_status;
					var nodetype = nodes[i].node_type;
					
					//if node inventory is not successful or user aborted then done show it in review
					if(op_status != 'done')
						continue;
					
					if(hasAnyTpm())
					    $('#tpm-status-update').show();
					else
					    $('#tpm-status-update').hide();
					
        			//add collapsible for each node 
					//add table for each node to show all the applicable components
					$('#all-installables').append('<div class="hp-collapsible" id="hpsum-otu-node-' + nodeid + '-installset"><label><b>' + nodes[i].node_name + ' ' + localizer.getString("hpsum.oneTouchUpdate.applicableComponents") + '</b></label><div>');
					
					$('#hpsum-otu-node-' + nodeid + '-installset').append('<div id="hpsum-otu-node-' + nodeid + '-installable"></div>');
					var node_Item = { node_id: nodeid, type: nodetype }
					var options = {
						command:'init',
						nodeItem:node_Item,
						onSelectionChanged:onSelectionChanged
					};
					
					//call Installables plug-in 
					$('#hpsum-otu-node-' + nodeid + '-installable').hpsumInstallables(options);
					
					$('#hpsum-otu-node-' + nodeid + '-installset').hpCollapsible();
					$('#hpsum-otu-node-' + nodeid + '-installset').hpCollapsible("expand");
					}
				}
				
				/*
			 *  initNodeSettings() : Initialize node settings for each node
			 *  Calls getdata to retrieve tpm_state and getattributes to retrieve tpm_bypass settings 
			 */
			function initNodeSettings() {
				
				var dataDeferred = $.Deferred();
				var optionsDeferred = $.Deferred();
				
				//hide the tpm status on startup
				$('#tpm-status-update').hide();
				$('#all-installables').empty();
				
				var oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
        		var nodes = [];
        		
        		if(oNodeTable)
        			nodes = oNodeTable.fnGetData();
        			
        		if(gnodeNodeSettings.length <= 0)
        		{
        		    //get the tpm state and tpm bypass setting for each node 
	        		for(var i=0; i < nodes.length; i++)
	        		{
	        			var list = nodes[i].node_uri.split("/");
						var nodeid = list[4];
						var op_status = nodes[i].op_status;
						var nodetype = nodes[i].node_type;
						
						//if node inventory is not successful or user aborted then done show it in review
						if(op_status != 'done')
							continue;
						
						var nodeItem = { node_id: nodeid };
						
						var nodeSetting = { node_id: nodeid, nodename: '', tpm_state: false, tpm_bypass: false, tpm_state_done: false, tpm_bypass_done: false};
						
						gnodeNodeSettings.push(nodeSetting);
						
						//getdata for the node and store tpm value
						NodePresenter.getNodeData(nodeItem, {
								nodeid: nodeid,
								success : function(data) {
									
									if(gnodeNodeSettings.length <= 0){
										dataDeferred.resolve();
										return;
									}
										
									var gnodeInd=0;
									for(gnodeInd=0; gnodeInd < gnodeNodeSettings.length; gnodeInd++)
									{
										if(gnodeNodeSettings[gnodeInd].node_id == this.nodeid)
											break;
									}
									
									if(data.hapi.hasOwnProperty('tpm_state'))
									{
										if(data.hapi.tpm_state == true)
											gnodeNodeSettings[gnodeInd].tpm_state = true;
										else
											gnodeNodeSettings[gnodeInd].tpm_state = false;
									}
									else
										gnodeNodeSettings[gnodeInd].tpm_state = false;
									
									gnodeNodeSettings[gnodeInd].nodename = data.hapi.name;	
									gnodeNodeSettings[gnodeInd].tpm_state_done = true;
									
									dataDeferred.resolve();
									
								},
								error : function(errorInfo) {
									dataDeferred.resolve();
									
								}
						});
						
						//getattributes for the node and store tpm bypass value
						NodePresenter.otheroption(nodeItem, {
								nodeid: nodeid,
								success : function(data) {
									
									if(gnodeNodeSettings.length <= 0){
										optionsDeferred.resolve();
										return;
									}
										
									var gnodeInd=0;
									for(gnodeInd=0; gnodeInd < gnodeNodeSettings.length; gnodeInd++)
									{
										if(gnodeNodeSettings[gnodeInd].node_id == this.nodeid)
											break;
									}
									
									if(data.hapi.Server_settings.hasOwnProperty('tpm_bypass'))
									{
										if(data.hapi.Server_settings.tpm_bypass == true)
											gnodeNodeSettings[gnodeInd].tpm_bypass = true;
										else
											gnodeNodeSettings[gnodeInd].tpm_bypass = false;
									}
									else
										gnodeNodeSettings[gnodeInd].tpm_bypass = false;
										
									gnodeNodeSettings[gnodeInd].tpm_bypass_done = true;
									
									optionsDeferred.resolve();
								},
								error : function(errorInfo) {
									optionsDeferred.resolve();
								}
						});
					}
				}
				else
				{
					//getnodedata and otheroption rest calls are already done previously, hence no need call these calls again
					dataDeferred.resolve();
					optionsDeferred.resolve();
				}
				
				$.when(dataDeferred, optionsDeferred).done(function(){
					
					//show TPM warning message to user if applicable
	    			if(hasAnyTpm()) {
	    				showTpmMessage();
	    			}
	    			else {
	    				//move ahead if TPM is not set to any node
	    				
	    				//dont show install review and go directly to deploy if non-interactive mode
	                    if(bModeNonInteractive) {
	                    	$("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
	                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
	                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step-selected");
	                        showWizardStep(STEP_DEPLOY);
			            	hideWizardStep(STEP_REVIEW);
			            	hideWizardStep(STEP_INVENTORY);
			            	initDeploy();
	            		}
	        			else {
	        				//show review screen with applicable components to user
	                		initInstallReview(false);
	                	}
	    			}	
				});
			}
			
			function logOffSession(handlers) {

				var closeHandlers = { 
					success:function()
					{
						session.eraseAll();
						if (handlers && handlers.success) {
							handlers.success({});
						}
						window.document.write("HPSUM Engine is Shutdown. Please restart the engine to Login.");						
					},
					error:function(){
						Log.error("Error shutting down the engine");
					}
				};
		
				session.shutDown(closeHandlers);
				Log.info("initiated shutdown");
				closeBrowser();				
			}			
			/*
			* closeBrowser() - Closes browser (tab)
			*/
			function closeBrowser() 
			{			
				window.open('', '_self', ''); 
				window.close();
			}
			
			function shutdownService() 
			{			
				var timeOut = setTimeout(function(){logOffSession()},10000);				
			}
			
			function handleExpressExit()
			{
				var guimode = session.getProperty("gui_mode");
				if(guimode === "express" ) {
					shutdownService();
				}
			}
			/*
			 * showTpmMessage() - Display TPM message in the review page
			 */
			function showTpmMessage() {
				
				$('#tpm-status-update').show();
				
				//in case of automatic mode, show deploy button to proceed further
				if(bModeNonInteractive) {
					showWizardStep(STEP_REVIEW);
            		hideWizardStep(STEP_INVENTORY);
            		hideWizardStep(STEP_DEPLOY);
	    			showDeployButton(STEP_REVIEW);
	    		}
				else {
					disableAnalyzeButton(STEP_REVIEW);
					//show review screen with applicable components to user
	        		initInstallReview(false);
	        	}	
			}
			
			/*
			 * showTpmMessageDialog() - Display TPM message confirmation dialog
			 * This is unused function as we are not using dialog to show TPM setting 
			 */
			function showTpmMessageDialog() {
				
				$('#hpsum-confirm-dialog-node', confirmDialogContents).html(getTpmNodes());
				
				new DialogView({contents: confirmDialogContents,
		                    ok: function (elem) {
		                        //dont show install review and go directly to deploy if non-interactive mode
		                        if(bModeNonInteractive) {
			            			$("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step");
			                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
			                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step-selected");
			                        
			                        initDeploy();
			            		}
		            			else {
									//show review screen with applicable components to user
	                        		initInstallReview(false);
	                        	}
		                    	return true;
		                    },
		                    cancel: function () {
		                    	//move to step1 and stop there
		                    	$("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step-selected");
		                        $("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
		                        $("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step");
		                        showWizardStep(STEP_INVENTORY);
				            	hideWizardStep(STEP_REVIEW);
				            	hideWizardStep(STEP_DEPLOY);
				            	
				            	//Disable next button by default till inventory is done
				            	enableNextButton(STEP_INVENTORY);
				            	//disable abort button, enable when inventory is in progress
				            	disableAbortButton(STEP_INVENTORY);
                        
		                    	return false;
		                    }
		    	});
			}
			
			/*
			 * hasAnyTpm: Checks whether a node has TPM value set from global gnodeNodeSettings
			 * Return true if atleaset any one of the node has tpm value set, otherwise return false 
			 */
			function hasAnyTpm() {
				
				if(gnodeNodeSettings.length <= 0)
					return false;
				
				var bTpmSet = false;
					
				var oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
        		var nodes = [];
        		
        		if(oNodeTable)
        			nodes = oNodeTable.fnGetData();
	            
	            //get the tpm state and tpm bypass setting for each node 
        		for(var i=0; i < nodes.length; i++)
        		{
        			var list = nodes[i].node_uri.split("/");
					var nodeid = list[4];
					var op_status = nodes[i].op_status;
					var nodetype = nodes[i].node_type;
					
					//if node inventory is not successful or user aborted then done show it in review
					if(op_status != 'done')
						continue;
									
					if(isTpmSet(nodeid)) {
						bTpmSet = true;
						break;
					}
				}
				
				return bTpmSet;
			}
			
			/*
			 * isTpmSet: Checks whether a node has TPM value set from global gnodeNodeSettings
			 * return True if a particular node has tpm value set, otherwise return False.
			 */
			function isTpmSet(nodeid) {
				
				var gnodeInd=0, bReturn=false;
				
				for(gnodeInd=0; gnodeInd < gnodeNodeSettings.length; gnodeInd++)
				{
					if(gnodeNodeSettings[gnodeInd].node_id == nodeid && gnodeNodeSettings[gnodeInd].tpm_state == true && gnodeNodeSettings[gnodeInd].tpm_bypass == false)
					{
						bReturn = true;
						break;		
					}
				}
				
				return bReturn;
			}
			
			/*
			 * getTpmNodes: Return all nodes which has tpm value set
			 * 
			 */
			function getTpmNodes() {
				
				var gnodeInd=0;
				var list = [];
				
				for(gnodeInd=0; gnodeInd < gnodeNodeSettings.length; gnodeInd++)
				{
					if(gnodeNodeSettings[gnodeInd].tpm_state == true)
					{
						list.push(gnodeNodeSettings[gnodeInd].nodename);		
					}
				}
				
				return list;
			}
				
				function recalculateInstallReview() {
				
					var nodes = [];
					var oNodeTable;
					var NodesDoneCount=0;
					
					showAnalysisSplash();
				
					var progressTimer = setInterval(function () {
						
						oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
							
						if(oNodeTable)
							nodes = oNodeTable.fnGetData();
						
						//start inventory for each baseline
						for(var i=0; i < nodes.length; i++)
						{
							var uri = nodes[i].node_uri;
							var vals = uri.split("/");
							var nodeid = vals[4];
							var index = i;
							var op_status = nodes[i].op_status;
							
							//check whether the current node status is UAN before calling getupdatelist.. need to make sure node is not still in analysis phase		
							rest.getURI(uri + "/getstatus", {
									row: index,
									node_id: nodeid,
									success : function(data) {
										
										if(data.hapi.node_status.node_state === "ANALYSIS")
										{
											
										}
										else if(data.hapi.node_status.node_state === "UAN")
										{
											NodesDoneCount++;
										}
										else if(data.hapi.node_status.node_state === "ERROR")
										{
											NodesDoneCount++;
										}
										else if(data.hapi.node_status.node_state === "PENDING_FW_UPDATE")
										{
											NodesDoneCount++;
										}
									},
									error : function(errorInfo) {
										
									}
							});
								
						}
							
						if(NodesDoneCount >= nodes.length) {
							
							//stop the timer
							clearInterval(progressTimer);
							
							//show install review for all the nodes
							initInstallReview(false);
							
							closeAnalysisSplash();
							
							//disable analyze button
							disableAnalyzeButton(STEP_REVIEW);
						}
							
					}, 3000);
					
				}
            function toggleButtons(nodeid, step)
            {
            	var tableid = '#hpsum-otu-installables-' + nodeid + '-table';
            	var currTable = $(tableid).dataTable();
	 			
	 			var selectedRows = $(tableid + ' tbody tr');
	 			
	 			var bselected = true, bforce = false, selectedCount=0;
	 			
	           	for(var tmp=0; tmp < selectedRows.length; tmp++){
	 				var compData = currTable.fnGetData(selectedRows[tmp]._DT_RowIndex);
	 				
	 				if(selectedRows[tmp].className.indexOf('hp-selected') != -1)
	 				{
						selectedCount++;
						if(compData.failed_deps.length > 0) {
							bselected = false;
							break;
						}
	 				}
	 			}
	 			
	 			if(bselected && selectedCount > 0)
	 			{
	 				enableNextButton(step);
	 				enableAnalyzeButton(step);
	 			}
	 			else
	 			{
	 				disableNextButton(step);
	 				disableAnalyzeButton(step);
	 			}
            }
            
            function initDeploy()
            {
            	var oNodeTable = $(INVENTORY_NODE_TABLE).dataTable();
	            		
        		var nodes = [];
        		var uri, index, op_status;
        		        		        		
        		var oTable = $(INSTALLPROGRESS_TABLE).dataTable();
				
				showWizardStep(STEP_DEPLOY);
				hideWizardStep(STEP_REVIEW);
				hideWizardStep(STEP_INVENTORY);
        		
        		if(bModeNonInteractive)
        		{
        			hideAllButtons(STEP_DEPLOY);
        		}
        		else
        		{
        			showAllButtons(STEP_DEPLOY);
        			disableBackButton(STEP_DEPLOY);
        		}
        		
        		disableRebootButton(STEP_DEPLOY);
        		
        		oTable.fnClearTable();
        		
        		//clear if any static or previous install results
        		$("#all-installresults").empty();
        		
        		if(oNodeTable)
        			nodes = oNodeTable.fnGetData();
        			
        		var installDoneCount=0;
	            //start deploy for each node
	            //later this needs to be enhanced to pass selected components and other install options to the deploy command
	            for(var i=0; i < nodes.length; i++)
        		{
        			var list = nodes[i].node_uri.split("/");
					var nodeid = list[4];
					
					//if node is in error state, then dont add it for deploy
					//if node inventory is not successful or user aborted then dont show it in review
					if(nodes[i].op_status != "done")
						continue;
					
        			//add node into the install progress table
        			var item = [{status: '<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>', 
                			node_name: nodes[i].node_name,
                			node_status: localizer.getString("hpsum.oneTouchUpdate.deploymentInProgress"),
                			node_progress: '<div id="install-progress-' + nodeid + '" style="height:15px"></div>',
                			node_details: '',
                			node_uri: nodes[i].node_uri,
                			op_status: "inprogress", //"started",
                			node_id: nodeid,
                			progress: 0}];
				  	  
					//add node into node inventory table of step1 of wizard	    
					oTable.fnAddData(item);
					
					$("#install-progress-" + nodeid).hpProgressBar(5);
        			
        			var index = i;
        			var uri = nodes[i].node_uri;
        			
        			//start deploy
        			rest.getURI(nodes[i].node_uri + "/deploy", 
    							{
    								row: index,
    				                node_id: nodeid,  
	                    			success : function(data) {
	                    				
						               enableAbortButton(STEP_DEPLOY);
									   /* Disable the Actions Drop Down Menu when the Deployment is in progress */
									   $('#hpsum-otu-actions').addClass('hp-disabled');
						                
	                        		},
	                    			error : function(errorInfo) {
	                        			handleExpressExit();
	                    			}
	                    		});
		        }
		        
		        var NodesDoneCount=0;
		        
		        var progressTimer = setInterval(function () {
		        	
		        	var nodesAdded;
		        	var oProgressTable = $(INSTALLPROGRESS_TABLE).dataTable();
	            		
	        		if(oProgressTable)
	        			nodesAdded = oProgressTable.fnGetData();
		        	
		        	//start inventory for each baseline
            		for(var i=0; i < nodesAdded.length; i++)
            		{
            			var uri = nodesAdded[i].node_uri;
            			var vals = uri.split("/");
            			var nodeid = vals[4];
            			var index = i;
            			var op_status = nodesAdded[i].op_status;
            			var currprogress = nodesAdded[i].progress;
            			//if current operation status is started, then only start the inventory, otherwise no need to start inventory
            			if(op_status === "inprogress") {
	            				
				        	rest.getURI(uri + "/getstatus", {
		                			row: index,
		                			node_id: nodeid,
		                			progress: currprogress,
		                			success : function(data) {
		                				
		                				if(data.hapi.node_status.node_state === "INSTALL")
		                				{
		                					if(this.progress < 95) 
	                    					{
	                    						$('#install-progress-' + this.node_id).hpProgressBar(this.progress+2);
		                    					oProgressTable.fnUpdate(this.progress+2, this.row, 8);
		                    				}
		                    				
						                	oProgressTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.deploymentInProgress"), this.row, 2)
						                	oProgressTable.fnUpdate(data.hapi.node_status.node_status, this.row, 4)
						                	oProgressTable.fnUpdate('inprogress', this.row, 6);
		                				}
		                				else if(data.hapi.node_status.node_state === "INSTALLDONE")
		                				{
		                					$('#install-progress-' + this.node_id).hpProgressBar(100);
						                	oProgressTable.fnUpdate(100, this.row, 8);
						                	
		                					oProgressTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>', this.row, 0);
		                					oProgressTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.deploymentCompleted"), this.row, 2);
		                					oProgressTable.fnUpdate(data.hapi.node_status.node_status, this.row, 4)
		                					oProgressTable.fnUpdate('done', this.row, 6);
		                					
		                					NodesDoneCount++;
		                				}
		                				else if(data.hapi.node_status.node_state === "ERROR")
		                				{
		                					$('#install-progress-' + this.node_id).hpProgressBar(100);
		                					oProgressTable.fnUpdate(100, this.row, 8);
						                	
		                					oProgressTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', this.row, 0);
		                					oProgressTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.deploymentError"), this.row, 2);
		                					oProgressTable.fnUpdate(data.hapi.node_status.node_status, this.row, 4)
		                					oProgressTable.fnUpdate('done', this.row, 6);
		                					
		                					NodesDoneCount++;
		            					}
		            					else
		            					{
		            						//for any other state, consider as done with error
		            						$('#install-progress-' + this.node_id).hpProgressBar(100);
		                					oProgressTable.fnUpdate(100, this.row, 8);
						                	
		                					oProgressTable.fnUpdate('<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>', this.row, 0);
		                					oProgressTable.fnUpdate(localizer.getString("hpsum.oneTouchUpdate.deploymentError"), this.row, 2);
		                					oProgressTable.fnUpdate('', this.row, 4)
		                					oProgressTable.fnUpdate('done', this.row, 6);
		                					
		                					NodesDoneCount++;	
		            					}
		            			 	},
		                			error : function(errorInfo) {
								handleExpressExit();
		                			}
		            			});
		            		}
		            	}
		            	
		            	if(NodesDoneCount >= nodesAdded.length) {
		            		
		            		//stop the timer
	            			clearInterval(progressTimer);
	            			
	            			$('#hpsum-otu-advance-action').addClass('hp-disabled');
	            			
	            			showAllButtons(STEP_DEPLOY);
	            			enableBackButton(STEP_DEPLOY);
		                			                	
		                	disableAbortButton(STEP_DEPLOY);
		                	enableRebootButton(STEP_DEPLOY);
							/* Enable the Actions Drop Down Menu when the deployment has been completed */
							$('#hpsum-otu-actions').removeClass('hp-disabled');
		                	
		                	//show deploy results for all the nodes
					        for(var i=0; i < nodesAdded.length; i++)
			        		{
			        			initDeployResults(nodesAdded[i].node_id, nodesAdded[i].node_name);
			        		}
					        
					        bInProgress = false;
							//if offline automatic mode, then reboot the server
							var guimode = session.getProperty("gui_mode");
							if(guimode === "offline_automatic" ) {
								// Just shut down HP SUM and let the boot environment take care of rebooting the server
								var handlers = { success:function() {
									var newWindow = window.open('', '_self', '');
									newWindow.close();
								}, 
									error: function(){
											   var newWindow = window.open('', '_self', '');
											   newWindow.close();
											   Log.error("Failed to shutdown HP Smart Update Manager");
										   }
								};
								session.shutDown(handlers);
							}
							handleExpressExit();
		            	}
            			
		        }, 3000);
		        
		    }
            
            function onCompGetLogsSuccess(data) {
            	showComponentDialog(data);
            }
            
            function onCompGetLogsError(error) {
            }
            
            function showComponentDialog(data) {
            	
            	//show component dialog
            	var dialog = new DialogView({contents: componentLogDialogContents,
                    ok: function (elem) {
                        return true;
                    },
                    cancel: function () {
                    }
                });
                
                if(data.hapi.components.component[0].hasOwnProperty('failed_deps'))
                {
                    $('#component_title').html(localizer.getString("hpsum.oneTouchUpdate.componentDetails"));
	                if(data.hapi.components.component.length > 0)
	                {
	                	$('#hpsum-comp-fail-dep').show();
	                	$('#hpsum-comp-inst-notes').hide();
		                $('#hpsum-component-id').html(data.hapi.components.component[0].component_id);
						
						
		                initFailedDependencyTable('#hpsum-component-failed-dep');
		                
		                var oFailedDepTable = $('#hpsum-component-failed-dep').dataTable();
		                
		                if(oFailedDepTable) {
							oFailedDepTable.fnClearTable();
		                	for(var i=0; i < data.hapi.components.component[0].failed_deps.length; i++)
		                	{
		                		var item =  [
		                						{
		                							failed_dependency: data.hapi.components.component[0].failed_deps[i]
				                				}
				                			];
		                		oFailedDepTable.fnAddData(item);
		                	}
		                }
		                
		                //this is for textarea
		                //$('#hpsum-component-failed-dep').val(data.hapi.components.component[0].failed_deps);
		                
						if(data.hapi.components.component[0].hasOwnProperty('device_info') && data.hapi.components.component[0].device_info.hasOwnProperty('devices'))
						{
							$('#hpsum-device-list-div').empty();
							
							$('#hpsum-device-list').show();
							
							for(var devInd=0; devInd < data.hapi.components.component[0].device_info.devices.device.length; devInd++)
							{
								var device = data.hapi.components.component[0].device_info.devices.device[devInd];
								
								//replace white spaces with - as element id should not have white spaces
								var headertableid = 'hpsum-devices-header-table-' +  device.device_id.replace(/\s+/g, '-').replace(/\./g, '_');
								$('#hpsum-device-list-div').append('<table id="' + headertableid + '"><thead><tr><td></td><td></td><td></td></tr></thead></table>');
								
								initDevicesHeaderTable('#' + headertableid);
								
								var headerTable = $('#' + headertableid).dataTable();
								
								var item = [{deviceid: device.device_id, 
						                	productid: device.product_id 
				                			}];
						                			
								headerTable.fnAddData(item);
								
								//replace white spaces with - as element id should not have white spaces
								var rowtableid = 'hpsum-devices-rows-table-' +  device.device_id.replace(/\s+/g, '-').replace(/\./g, '_'); 
								$('#hpsum-device-list-div').append('<table class="hp-selectable" id="' + rowtableid + '"><thead><tr><td>' + localizer.getString("hpsum.common.type") + '</td><td>' + localizer.getString("hpsum.common.statusStr") + '</td><td>' + localizer.getString("hpsum.node.activeVersion") + '</td><td>' + localizer.getString("hpsum.node.availableVersion") + '</td></tr></thead></table>');
								
								initDevicesRowTable('#' + rowtableid);
								
								var rowTable = $('#' + rowtableid).dataTable();
								
								for(var fwInd=0; fwInd < device.fw_item.length; fwInd++)
								{
									var fwitem = device.fw_item[fwInd];
									
									var rowitem = [{type: fwitem.type, 
							                	fw_item_status: fwitem.fw_item_status,
							                	active_version: fwitem.active_version,
							                	version: fwitem.version 
					                			}];
							                			
									rowTable.fnAddData(rowitem);
								}
								
								$('#' + rowtableid + " tbody tr").click( function( e ) {
			                   		
									var data = rowTable.fnGetData( this );
									
									//toggle row selection	
							        $(this).toggleClass('hp-selected');
							    });
							}
						}
						else
						{
							$('#hpsum-device-list-div').empty();
							$('#hpsum-device-list').hide();
						}
		            }
		       	}
		       	else{
		       		
		       		//reset the component id and log text before showing
		       		$('#hpsum-component-id').html('');
		       		$('#hpsum-component-installation-notes').val('');
		       		
                    $('#component_title').html(localizer.getString("hpsum.oneTouchUpdate.deploymentLog"));
	                if(data.hapi.components.component.length > 0)
	                {
	                	$('#hpsum-comp-inst-notes').show();
	                	$('#hpsum-comp-fail-dep').hide();
	                	$('#hpsum-device-list').hide();
		                $('#hpsum-component-id').html(data.hapi.components.component[0].component_id);
		                $('#hpsum-component-installation-notes').val(data.hapi.components.component[0].component_log);
		            }
		       	}
	            
            }
            
            function initDeployResults(node_id, node_name)
            {
            	var nodes = [];
        		var uri, index, op_status;
        		var bNodesDoneCount=0;
				$('.hpsumInstallables').hpsumInstallables('destroy');
        		
        		var uri = "/Session/" + session.getToken() + "/node/" + node_id;
        		
        		var tableid = 'hpsum-otu-install-results-table-' + node_id;
        		$('#all-installresults').append('<div class="hp-collapsible" id="hpsum-otu-node-' + node_id + '-installresult"><label>' + node_name + ' - ' + localizer.getString('hpsum.oneTouchUpdate.deploymentResults') + '</label><div>');
        		
				
				$('#hpsum-otu-node-' + node_id + '-installresult').hpsumInstallResults();
                $('#hpsum-otu-node-' + node_id + '-installresult').hpsumInstallResults('viewLogHandler', { success:function(data){ 
                                                                                    onCompGetLogsSuccess(data); 
                                                                                },
                                                                            error:function(data){ 
                                                                                    onCompGetLogsError(data); 
                                                                                } 
                                                                         } );
                                                                         
                $('#hpsum-otu-node-' + node_id + '-installresult').hpsumInstallResults("setNodeId",node_id);
                $('#hpsum-otu-node-' + node_id + '-installresult').hpsumInstallResults("populateTable");
            }
            
            //Reset functionality
            function initReset() {
            	var oTable;
            	//clear all the tables
            	oTable = $(INVENTORY_BASELINE_TABLE).dataTable();
            	oTable.fnClearTable();
				$('.hpsumInstallables').hpsumInstallables('destroy');
            	
            	oTable = $(INVENTORY_NODE_TABLE).dataTable();
            	oTable.fnClearTable();
            	
            	oTable = $(INSTALLPROGRESS_TABLE).dataTable();
            	oTable.fnClearTable();
            	
            	//Initialize the wizard steps
            	showWizardStep(STEP_INVENTORY);
            	hideWizardStep(STEP_REVIEW);
            	hideWizardStep(STEP_DEPLOY);
				
				//re-initialize bread crumbs
				$("#hpsum-step1").attr("class", "hpsum-bread-crumbs-step-selected");
				$("#hpsum-step2").attr("class", "hpsum-bread-crumbs-step");
				$("#hpsum-step3").attr("class", "hpsum-bread-crumbs-step");
            	
            	bInProgress = false;
                abortButtonPressed = false;
            	
            	loadBaselines();
            	loadNodes();
            	
            	//disable Actions menu in the beginning
				$('#hpsum-otu-actions').hide();
		
		//reset global gnodeNodeSettings array
				gnodeNodeSettings = [];
            	
            	showActionDialog();
            }
            
            function onNodeOptionChange(){
            	
            	var data = [];
            	
            	if($(this).length <= 0)
            		return
            	
            	if($(this)[0].id === "node-localhost" && $(this)[0].checked === true) {
            		$('#hpsum-otu-action-node').hide();
            		$('#hpsum-otu-action-baseline').hide();
            	}
            	else if($(this)[0].id === "node-multiple" && $(this)[0].checked === true) {
            		
            		$('#hpsum-otu-action-node').show();
            		$('#hpsum-otu-action-baseline').show();
            	}
        		else if($(this)[0].id === "non-interactive-mode" && $(this)[0].checked === true) {
        			$(".optional-div").hide();
        		}
        		else if($(this)[0].id === "interactive-mode" && $(this)[0].checked === true) {
        			$(".optional-div").show();
        		}
            }
            
            function onModeChange(){
            	
            	if($(this).length <= 0)
            		return
            	
            	var nodeOption = $('input[name=node-option]:checked').val();
            	
            	if($(this)[0].id === "interactive-mode" && $(this)[0].checked === true) {
            		
            		if(nodeOption === "node-localhost") {
                		$('#hpsum-otu-action-node').hide();
                		$('#hpsum-otu-action-baseline').hide();
                	}
                	else {
                		$('#hpsum-otu-action-node').show();
                		$('#hpsum-otu-action-baseline').show();
                	}
					
				}
            	else if($(this)[0].id === "non-interactive-mode" && $(this)[0].checked === true) {
            		if(nodeOption === "node-localhost") {
                		$('#hpsum-otu-action-node').hide();
                		$('#hpsum-otu-action-baseline').hide();
                	}
                	else {
                		$('#hpsum-otu-action-node').show();
                		$('#hpsum-otu-action-baseline').show();
                	}
            	}
        	}
			
			function selectComponents() {
				if($('#chkboxheader').attr('checked')) 
				{
					$('.chkboxcomp').attr('checked', true);
				}
                else {
					$('.chkboxcomp').attr('checked', false);
				}
			}
            
            function changeLocation(location) {
                
            }
            
            function toggleExpansion(row) {
                if (row.hasClass('hp-expanded')) {
                    $('> td > .hp-collapser', row).removeClass('hp-active');
                    row.removeClass('hp-expanded');
                    row.next().hide();
                } else {
                    $('> td > .hp-collapser', row).addClass('hp-active');
                    row.addClass('hp-expanded');
                    if ($('.hp-plot', row.next()).children().length === 0) {
                        //initStatistics(row.next());
                    }
                    row.next().show();
                }
            }
            
            //Initializes baseline and node inventory tables
            function initInventoryTables() {
            	//Sample data added here to example for table data
				var data = [{status: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>', 
                			baseline_name: 'HP Service Pack for Proliant 2012',
                			baseline_status: 'Processing baseline inventory',
                			baseline_progress: '<div id="baseline-progress"></div>',
                			baseline_details: 'Total 499 components found',
                			baseline_uri: '/Session/session1/Baseline/location/bl1/bundle/bp00001',
                			op_status: 'started'}];
				    
				var oTab = $(INVENTORY_BASELINE_TABLE).dataTable({
				    bPaginate : false,
				    bProcessing : true,
				    bFilter : false,
				    bSort : false,
				    bAutoWidth : false,
				    bInfo : false,
				    bRetrieve : true,
				    bDestroy : true,
				    //aaData: data,
				    aoColumns : [
				        {mDataProp: 'status', sClass: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>'},
				        {mDataProp: 'baseline_name'},
				        {mDataProp: 'baseline_status'},
				        {mDataProp: 'baseline_progress', sClass: '<div id="baseline-progress"></div>'},
				        {mDataProp: 'baseline_details'},
				        {mDataProp: 'baseline_uri', "bVisible": false},
				        {mDataProp: 'op_status', "bVisible": false}],
				    aaSorting: [[ 0, "asc" ]],
				    fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
				        //$(nRow).attr('id', aData.component_file_name);
				        return nRow;
				    }
				    });//.rowReordering().addClass('hp-reorderable');
				 
				$(INVENTORY_BASELINE_TABLE + ' tbody tr').first().addClass('hp-selected');
								
				//Sample data added here to show node inventory progress
                var dataNode = [{status: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>', 
                			node_name: '127.0.0.1',
                			node_status: 'Inventory in progress',
                			node_progress: '<div id="node-progress"></div>',
                			node_details: 'Success or error',
                			node_uri: '/node/',
                			op_status: 'started',
                			progress: 10,
                			node_type: 'windows'}];
				    
				var oTab = $(INVENTORY_NODE_TABLE).dataTable({
				    bPaginate : false,
				    bProcessing : true,
				    bFilter : false,
				    bSort : false,
				    bAutoWidth : false,
				    bInfo : false,
				    bRetrieve : true,
				    bDestroy : true,
				    //aaData: dataNode,
				    aoColumns : [
				        {mDataProp: 'status', sClass: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>'},
				        {mDataProp: 'node_name'},
				        {mDataProp: 'node_status'},
				        {mDataProp: 'node_progress', sClass: '<div id="node-progress"></div>'},
				        {mDataProp: 'node_details'},
				        {mDataProp: 'node_uri', "bVisible": false},
				        {mDataProp: 'op_status', "bVisible": false},
				        {mDataProp: 'progress', "bVisible": false},
				        {mDataProp: 'node_type', "bVisible": false}],
				    aaSorting: [[ 0, "asc" ]],
				    fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
				        //$(nRow).attr('id', aData.component_file_name);
				        return nRow;
				    }
				    });
				 
				$(INVENTORY_NODE_TABLE + ' tbody tr').first().addClass('hp-selected');
            }
            
            //Initialize component device info row table 
            function initFailedDependencyTable(tableid) {
            	
            	var oTab = $(tableid).dataTable({
					oLanguage: {
								sEmptyTable: ""
							   },
				    bPaginate : false,
				    bProcessing : false,
				    bFilter : false,
				    bSort : false,
				    bAutoWidth : false,
				    bInfo : false,
				    bRetrieve : true,
				    bDestroy : true,
				    aoColumns : [
				    	{mDataProp: 'failed_dependency'}
				        ],
				    aaSorting: [],
				    fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
				        return nRow;
				    }
				    });
			 
				$(tableid + ' tbody tr').first().addClass('hp-selected');
			}
            
            //Initialize component device info row table 
            function initDevicesHeaderTable(tableid) {
            	
            	var oTab = $(tableid).dataTable({
					oLanguage: {
								sEmptyTable: ""
							   },
				    bPaginate : false,
				    bProcessing : false,
				    bFilter : false,
				    bSort : false,
				    bAutoWidth : false,
				    bInfo : false,
				    bRetrieve : true,
				    bDestroy : true,
				    aoColumns : [
				    	{mDataProp: 'deviceid'},
				    	{mDataProp: 'productid'}
				        ],
				    aaSorting: [],
				    fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
				        return nRow;
				    }
				    });
			 
				$(tableid + ' tbody tr').first().addClass('hp-selected');
			}
			
			//Initialize component device info row table 
            function initDevicesRowTable(tableid) {
            	
            	var oTab = $(tableid).dataTable({
					oLanguage: {
								sEmptyTable: ""
							   },
				    bPaginate : false,
				    bProcessing : false,
				    bFilter : false,
				    bSort : false,
				    bAutoWidth : false,
				    bInfo : false,
				    bRetrieve : true,
				    bDestroy : true,
				    aoColumns : [
				    	{mDataProp: 'type'},
				    	{mDataProp: 'fw_item_status'},
				    	{mDataProp: 'active_version'},
				    	{mDataProp: 'version'}
				        ],
				    aaSorting: [],
				    fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
				        return nRow;
				    }
				    });
			 
				$(tableid + ' tbody tr').first().addClass('hp-selected');
			}
            
            function initDeployProgressTable() {
            	
            	var data = [{status: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>',
            				node_name: '127.0.0.1',
            				node_status: 'Deployment in progress 3/25', 
                			node_progress: '<div id="install-progress"></div>',
                			node_details: 'Deployment in progress 3/25',
                			node_uri: 'node/localhost',
                			op_status: 'started',
                			node_id: 'N127h0h0h1',
                			progress: 0}];
				    
				var oTab = $(INSTALLPROGRESS_TABLE).dataTable({
					bPaginate : false,
				    bProcessing : true,
				    bFilter : false,
				    bSort : false,
				    bAutoWidth : false,
				    bInfo : false,
				    bRetrieve : true,
				    bDestroy : true,
				    //aaData: data,
				    aoColumns : [
				        {mDataProp: 'status', sClass: '<div class="hp-status hp-changing"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div><div class="hp-status-changing"></div></div>'},
				        {mDataProp: 'node_name'},
				        {mDataProp: 'node_status'},
				        {mDataProp: 'node_progress', sClass: '<div id="install-progress"></div>'},
				        {mDataProp: 'node_details'},
				        {mDataProp: 'node_uri', "bVisible": false},
				        {mDataProp: 'op_status', "bVisible": false},
				        {mDataProp: 'node_id', "bVisible": false},
				        {mDataProp: 'progress', "bVisible": false}],
				    aaSorting: [[ 0, "asc" ]],
				    fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
				        //$(nRow).attr('id', aData.component_file_name);
				        return nRow;
				    }
				    });
			 
				$(INSTALLPROGRESS_TABLE + ' tbody tr').first().addClass('hp-selected');
            	
            }
            
            function initDeployResultsTable(tableid) {
            	
            	var oTab = $(tableid).dataTable({
					oLanguage: {
								sEmptyTable: localizer.getString("hpsum.node.noApplicableComponentsFound")
							   },
				    bPaginate : false,
				    bProcessing : true,
				    bRetrieve : true,
				    bDestroy : true,
				    bFilter : true,
				    bSort : true,
				    bAutoWidth : false,
				    bInfo : false,
				    //aaData: data,
				    aoColumns : [
				        {mDataProp: 'status', sClass: '<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>'},
				        {mDataProp: 'component_id'},
				        {mDataProp: 'name'},
				        {mDataProp: 'installation_result'},
				        {mDataProp: 'view_log', "bVisible": false},
				        {mDataProp: 'node_uri', "bVisible": false},
				        {mDataProp: 'op_status', "bVisible": false}],
				    aaSorting: [[ 0, "asc" ]],
				    fnRowCallback : function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
				        //$(nRow).attr('id', aData.component_file_name);
				        return nRow;
				    }
				    });
			 
				$(tableid + ' tbody tr').first().addClass('hp-selected');
            	
            	$(tableid + " tbody tr").click( function( e ) {
					var data = oTab.fnGetData( this );
        		});
            }
            
            /**
             * @public
             */
            this.pause = function () {
            	//$('.hpsumInstallables').hpsumInstallables('destroy');
            };
            
            this.resume = function () {
                if(bInProgress == false)
                {
                	//load baselines list. This is used to display in action dialog baselines table 
                	loadBaselines();
                
                	//load nodes list. This is used to display in action dialog nodes table
                	loadNodes();
                	
                	//disable Actions menu in the beginning
					$('#hpsum-otu-actions').hide();
					
			//reset global gnodeNodeSettings array
					gnodeNodeSettings = [];
                
					showActionDialog();
					
					bStartDeploy = false;
				}
					else
					{	
						if(window.bRefreshReview == true && $('#hpsum-step2')[0].className == "hpsum-bread-crumbs-step-selected")
						{
							recalculateInstallReview();
							window.bRefreshReview = false;
						}
					}
            };
            
            this.init = function () {
		window.bRefreshReview = false;
            	$(DELTA_BASE).hpSelect();
                document.title = localizer.getString("hpsum.oneTouchUpdate.title") ;
                
            	$('.hp-details-actions').hpDropMenu();
				
				$('#hpsum-otu-action-node').hide();
				$('#hpsum-otu-action-baseline').hide();
				
				//Disable Actions until the Inventory finishes
				$('#hpsum-otu-actions').hide();
				
				actionDialogContents = $(ACTION_DIALOG).detach().show();
				autoConfDialogContents = $(AUTOMATIC_UPDATE_CONFIRM).detach().show();
				
				//Initialize wizard tables
				initInventoryTables();
                initDeployProgressTable();
                
                //load baselines list. This is used to display in action dialog baselines table 
                loadBaselines();
                //load nodes list. This is used to display in action dialog nodes table
                loadNodes();
                
                //Initialize action dialog tables
                initActionNodeTable();
				initActionBaselineTable();
				
				$(ADD_BASELINES, actionDialogContents).click(onAddBaselines);
            	$(ADD_NODES, actionDialogContents).click(onAddNodes);
            	
				var setting = {
				    ctrls: $("#wizard section"), //Structure for Steps
				        deployButtonTitle: localizer.getString("hpsum.node.doDeploy"),
		                nextButtonTitle: localizer.getString("hpsum.common.next"), 
		                nextButtonCss: 'hp-button hp-primary',
		                callBack: btnNextClicked, 
		                previousButtonTitle: localizer.getString("hpsum.common.back"), 
						callBackPrev: btnPrevClicked,
						previousButtonCss: 'hp-button hp-secondary',
						finishButtonTitle: localizer.getString("hpsum.oneTouchUpdate.reboot"), 
						callFinish: btnFinishClicked,
						finishButtonCss: 'hp-button hp-primary',
						abortButtonTitle: localizer.getString("hpsum.common.abort"),
						callAbort: btnAbortClicked,
						abortButtonCss: 'hp-button hp-secondary',
						resetButtonTitle: localizer.getString("hpsum.common.startOver"),
						callReset: btnResetClicked,
						resetButtonCss: 'hp-button hp-secondary',
						analyzeButtonTitle: localizer.getString("hpsum.oneTouchUpdate.analyze"), 
						callAnalyze: btnAnalyzeClicked,
						analyzeButtonCss: 'hp-button hp-secondary',
						validateWizardStep: validateStep,
		                validateWizardByTitle: validateByTitle,
		                initStep: '0'
		    	};
		        				
				showWizardScreen(setting);
				
				//////////////////////////////////////////////////////////////////////////////////////
				//The following lines are commented for HPSUM 6.0.0 as we are not supporting multiple nodes/baselines options
				//This code avoid showing of Nodes and Baseliens tables for selection
				//These lines can be uncommented in future release when we need multiple nodes/baselines support in Guided update
				//$('input:radio[name=node-option]', actionDialogContents).click(onNodeOptionChange);
                //$('input:radio[name=mode-option]', actionDialogContents).click(onModeChange);
                //////////////////////////////////////////////////////////////////////////////////////
				
				//if offline interactive or automatic, then no need to show action dialog
				//one touch update shall happen with default values for local host and default working directory
				//get the property from guimode cookie
				var guimode = session.getProperty("gui_mode");
				 
				if(guimode === "offline_automatic" ||
					guimode === "offline_interactive" || guimode === "express")
				{
					
					if(guimode === "offline_automatic" || guimode === "express")
                      {
						hideAllButtons(STEP_INVENTORY);				
                      }
                      else if(guimode === "offline_interactive")
                      {
                         var handlers = { success:function() {
                                           var newWindow = window.open('', '_self', '');
                                           newWindow.close();
                                        }, 
                                        error: function(){
                                               var newWindow = window.open('', '_self', '');
                                               newWindow.close();
                                               Log.error("Failed to shutdown HP Smart Update Manager");
                                        }
                                       };
                         $("#hpsum-offline-exit").click(function(){
                                     session.shutDown(handlers);
                                   });
                       }

					startOneTouchUpdate();
				}
				else
				{
                   //We will not show exit button in online Local guide update
                    $('#hp-exit-control').hide();
					showActionDialog();
				}	
            	
            	$(".hp-collapsible").hpCollapsible();
                $(".hp-collapsible").hpCollapsible("expand");
                
                $('#chkboxheader').on('click', selectComponents);
                
                $(VIEW + ' .hp-status').hpStatus();
                $(VIEW).on('click', 'td > .hp-collapser', function (ev) {
                    toggleExpansion($(this).parents('tr').first());
                });
                
                componentLogDialogContents = $(COMPONENT_DIALOG).detach().show();
				confirmDialogContents = $(CONFIRM_DIALOG).detach().show();            	            			         	
            	
			};
        }

        return new OneTouchUpdateView();
    }());
    
    return OneTouchUpdateView;
});
