// (C) Copyright 2011-2012 Hewlett-Packard Development Company, L.P.

define(['hpsum/presenter/node/NodePresenter', 
    'hp/core/UrlFragment',
    'hp/core/Router',
    'hp/core/LinkTargetBuilder',
    'text!hpPages/core/no_details.html',
    'hp/core/Localizer',
	'hp/model/Session',
    'jquery',
    'hp/lib/jquery.hpGridPanels'],
function(presenter, urlFragment,router, linkTargetBuilder, noneHtml, localizer, session) { "use strict";

    var VcOverviewView = (function() {
      
        var ROOT = '#hpsum-vc-show-overview';
        var PANELS = '.hp-grid-panels',
        	DESCRIPTION = '#hpsum-vc-description',
        	TYPE ='#hpsum-vc-type',
        	BASELINE='#hpsum-vc-bundle',
        	VCDOMAIN='#hpsum-vc-domain',
        	OA = '#hpsum-vc-oa',
			ASSOC_DEVICES = '#hpsum-vc-assocdevices',
        	NO_ASSOC_DEVICES ='#hpsum-vc-no-devices',
        	INSTALLEDROM ='#hpsum-vc-installed-rom',
        	HEALTHSTATUS = '#hpsum-vc-health-status',
        	MATRIXENABLED = '#hpsum-vc-matrix-enabled',
        	MATRIXVERSION = '#hpsum-vc-matrix-version',
        	SPINNER = '#hpsum-node-add-spinner',
            INSTALL_LOGS = '#hpsum-vc-install-log';
      
        /**
         * Constructor
         */
        function VcOverviewView() {
          
            var noItem;
            var timer= null;
            var thisNode=null;
            
            function onItemChange(item) {
            	item = item.hapi;
            	thisNode =item;
                noItem.remove();
        		item.type = item.type.toLowerCase();
        		if(!router.location().match(item.type))
        			router.go((urlFragment.replaceView(router.location(), 'show/' + item.type + '/overview')));

                $('a.hp-button' , $(INSTALL_LOGS)).attr('href', '#/' + urlFragment.getPage(router.location()) + '/deploy-results');
                $(INSTALL_LOGS).hide();

        		if(item.node_state.match(/idle/i) && item.node_phase.match(/initialize/i)){
        			$('.hp-grid-panels').css({"display":"none"});
        			 $(SPINNER).show();
        			 presenter.startNodeStatus(item);
        		}
        		else {
        			$(SPINNER).hide();
					$(NO_ASSOC_DEVICES).hide();
					$(ASSOC_DEVICES).hide();
        			$('.hp-grid-panels').css({"display":""});
        			$('.hp-grid-panels').hpGridPanels('reset');     			
        			$(DESCRIPTION).text(item.description);
        			if(item.hasOwnProperty('vc_domain'))
        				$(VCDOMAIN).text(item.vc_domain);
        			else 
        				$(VCDOMAIN).text('');
        			if(item.hasOwnProperty('vc_installed_version')){
						$(NO_ASSOC_DEVICES).hide();
						$(ASSOC_DEVICES).show();
        				$(INSTALLEDROM).text(item.vc_installed_version);
        				$(HEALTHSTATUS).text(item.health_status);
						if(item.assoc_node_id){
							var assoc_node_url = '#/node/show/r/session/' + session.getToken() +'/node/' + item.assoc_node_id + '/getdata';
							$(OA).html ('<a href="' + assoc_node_url + '">' + item.oa_address+ '</a>');
						}
        				else{
							$(OA).text(item.oa_address);
        				}
        			}
        			else{
        				$(INSTALLEDROM).text('');
        				$(HEALTHSTATUS).text('');
						$(NO_ASSOC_DEVICES).show();
        				$(ASSOC_DEVICES).hide();
        			}
        			showVCModules(item);
        			if(item.alt_name == undefined)
        				$(TYPE).text(item.type);
        			else
        				$(TYPE).text(item.alt_name);
        			
        			if(item.matrix_enabled === undefined){
        				$(MATRIXVERSION).text('');
        				$(MATRIXENABLED).text('');
        			}
        			else {
        				$(MATRIXENABLED).text(item.matrix_enabled);
        			if(item.matrix_enabled.match(/no/i))
        				$(MATRIXVERSION).text("");
        			else
        				$(MATRIXVERSION).text(item.matrix_major_version+ '.' + item.matrix_minor_version + ' update '
        							+ item.matrix_update_version + '.' + item.matrix_subupdate_version);
        			}
        			var fwBaseline =presenter.getBaselineOrPkg(item.baselines)
        			if(fwBaseline){
        					var baseline_url = '#/baseline/show/r'+fwBaseline.uri +'/getBaselineData';
        					name = fwBaseline.name;
        					$(BASELINE).html('<a href="'+ baseline_url +'">'+name+ '</a>');
        				}
        			else
        				$(BASELINE).text(localizer.getString('hpsum.node.noBaselineDefined'));  

                    if(item.node_state.match(/installdone/i))
                        $(INSTALL_LOGS).show();

        		}
                $(PANELS).hpGridPanels('reset');
                $(ROOT).removeClass('hp-show-changing');
                $('.hp-dynamic', ROOT).removeClass('hp-unavailable');
            }
            
            function showVCModules(item){
            	if(item.devices === undefined)
            		return;
            	if(item.devices.device === undefined)
            		return;
            		
            	var modulesTable = $('#hpsum-vc-modules-table').dataTable();
                modulesTable.fnClearTable();
            		 
            	var i;
            	for ( i=0; i<item.devices.device.length; i++){
            		var tableItem = [{
            						devicenumber: item.devices.device[i].devicenumber,
            						enclosure: item.devices.device[i].enclosure,
            						type: item.devices.device[i].product_id,
            						active_version : item.devices.device[i].fw_item.active_version,
            						version: item.devices.device[i].fw_item.version,
            						health_status : item.devices.device[i].fw_item.health_status
            					}];
            		modulesTable.fnAddData(tableItem);
            	}
            }
            
            function onItemError(error) {
                $(ROOT).append(noItem);
            }
            
            function onSelectionChange(selection) {
                urlFragment.replaceHrefUris(ROOT, selection.uris);
                if ($(ROOT).hasClass('hp-show-init')) {
                    $(ROOT).removeClass('hp-show-init');
                } else {
                    $(ROOT).addClass('hp-show-changing');
                    $('#tour-server-bundle-item').addClass('hp-show-changing');
                }
            }
          
            /**
             * @public
             */
            this.resume = function() {
                $(PANELS).hpGridPanels('reset');
                presenter.on("selectionChange", onSelectionChange);
                presenter.on("itemError", onItemError);
                presenter.on("itemChange", onItemChange);
                
            };
            
            this.pause = function() {
                presenter.off("selectionChange", onSelectionChange);
                presenter.off("itemChange", onItemChange);
                presenter.off("itemError", onItemError);
                
            };
             
            this.init = function() {
            	if(!presenter.initDone)
            		presenter.init();
            	var modulesTable = $('#hpsum-vc-modules-table').dataTable({
				  		oLanguage: {
								sEmptyTable: localizer.getString('hpsum.node.noItemsFound')
							   },
				    bPaginate : false,
				    bProcessing : true,
				    bFilter : true,
				    bSort : true,
				    bAutoWidth : true,
				    bInfo : false,
				    bRetrieve : true,
				    bDestroy : true,
				    	aoColumns : [
				        	{mDataProp: 'devicenumber'},
				        	{mDataProp: 'enclosure'},
				        	{mDataProp: 'type'},
				        	{mDataProp: 'active_version'},
				        	{mDataProp: 'version'},
				        	{mDataProp: 'health_status'}
				        	],
				    aaSorting: [[ 0, "asc" ]],
				    });
                noItem = $(noneHtml);
                $(PANELS).hpGridPanels();
                this.resume();
            };
        }

        return VcOverviewView;
    }());
    
    return new VcOverviewView;
});
