#!/bin/sh
#set -x

datetime=`date +"%m-%d-%Y_%H-%M-%S"`
tarName="HPSUM_Logs_${datetime}.tar"
tmpFile=".directory_listing.txt"
hpsumDirListing="hpsum_directory_listing.txt"
syslogFile="syslog.log"
oldsyslogFile="OLDsyslog.log"

#determine which zip program to use, or none
gzip=`which gzip`
compress=`which compress`

if [ "x$gzip" != "x" ]; then
	zipper=$gzip
else
	if [ "x$compress" != "x" ]; then
		zipper=$compress
	else
		#no zip
		zipper="touch"
	fi
fi

echo "Gathering HPSUM logs..."
found=0

#listing of current dir and sub dir, usually dir running HPSUM, to capture files, dates/times
ls -l * > $tmpFile

if [ ! -f $tmpFile ];then 
    tarName="/tmp/HPSUM_Logs_${datetime}.tar"
    tmpFile="/tmp/.directory_listing.txt"
    hpsumDirListing="/tmp/hpsum_directory_listing.txt"
    syslogFile="/tmp/syslog.log"
    oldsyslogFile="/tmp/OLDsyslog.log"
    ls -l * > $tmpFile
fi

tar -cf $tarName $tmpFile
rm $tmpFile

#HPSUM directory listing 
ls -Rl /tmp/HPSUM > $hpsumDirListing
tar -rf $tarName $hpsumDirListing
rm $hpsumDirListing

#debug logs
if [ -d /tmp/HPSUM ]; then
  find /tmp/HPSUM -name "*.txt" -o -name "*.log" -o -name "*.xml" -o -name "*.trace" -o -name "*.pdb*" -o -name "*.ini" |
  while read file; do
    tar -rf $tarName $file
  done
  found=1
fi

#user logs
if [ -d /var/hp/log ]; then
  tar -rf $tarName /var/hp/log/*
  found=1
fi

#system logs
tail -10000 /var/adm/syslog/syslog.log > $syslogFile 2>/dev/null
if [ -f $syslogFile ]; then
	tar -rf $tarName $syslogFile
	rm $syslogFile
fi

tail -10000 /var/adm/syslog/OLDsyslog.log > $oldsyslogFile 2>/dev/null
if [ -f $oldsyslogFile ]; then
	tar -rf $tarName $oldsyslogFile
	rm $oldsyslogFile
fi

#zip if possible
if [ $found -eq 1 ]; then
  $zipper $tarName
  newName=`ls -1 $tarName*`
  echo "HPSUM logs are in $newName"
  exit 0
fi

rm $tarName
echo "No HPSUM logs found"
exit -1
